#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: opensearch
short_description: Creates OpenSearch or ElasticSearch domain
description:
  - Creates or modify a Amazon OpenSearch Service domain.
version_added: 4.0.0
author:
  - "Sebastien Rosset (@sebastien-rosset)"
options:
  state:
    description:
      - Creates or modifies an existing OpenSearch domain.
      - Deletes an OpenSearch domain.
    required: false
    type: str
    choices: ['present', 'absent']
    default: present
  domain_name:
    description:
      - The name of the Amazon OpenSearch/ElasticSearch Service domain.
      - Domain names are unique across the domains owned by an account within an AWS region.
    required: true
    type: str
  engine_version:
    description:
      ->
        The engine version to use. For example, 'ElasticSearch_7.10' or 'OpenSearch_1.1'.
      ->
        If the currently running version is not equal to I(engine_version),
        a cluster upgrade is triggered.
      ->
        It may not be possible to upgrade directly from the currently running version
        to I(engine_version). In that case, the upgrade is performed incrementally by
        upgrading to the highest compatible version, then repeat the operation until
        the cluster is running at the target version.
      ->
        The upgrade operation fails if there is no path from current version to I(engine_version).
      ->
        See OpenSearch documentation for upgrade compatibility.
    required: false
    type: str
  allow_intermediate_upgrades:
    description:
      - >
        If true, allow OpenSearch domain to be upgraded through one or more intermediate versions.
      - >
        If false, do not allow OpenSearch domain to be upgraded through intermediate versions.
        The upgrade operation fails if it's not possible to ugrade to I(engine_version) directly.
    required: false
    type: bool
    default: true
  cluster_config:
    description:
      - Parameters for the cluster configuration of an OpenSearch Service domain.
    type: dict
    suboptions:
      instance_type:
        description:
          - Type of the instances to use for the domain.
        required: false
        type: str
      instance_count:
        description:
          - Number of instances for the domain.
        required: false
        type: int
      zone_awareness:
        description:
          - A boolean value to indicate whether zone awareness is enabled.
        required: false
        type: bool
      availability_zone_count:
        description:
          - >
            An integer value to indicate the number of availability zones for a domain when zone awareness is enabled.
            This should be equal to number of subnets if VPC endpoints is enabled.
        required: false
        type: int
      dedicated_master:
        description:
          - A boolean value to indicate whether a dedicated master node is enabled.
        required: false
        type: bool
      dedicated_master_instance_type:
        description:
          - The instance type for a dedicated master node.
        required: false
        type: str
      dedicated_master_instance_count:
        description:
          - Total number of dedicated master nodes, active and on standby, for the domain.
        required: false
        type: int
      warm_enabled:
        description:
          - True to enable UltraWarm storage.
        required: false
        type: bool
      warm_type:
        description:
          - The instance type for the OpenSearch domain's warm nodes.
        required: false
        type: str
      warm_count:
        description:
          - The number of UltraWarm nodes in the domain.
        required: false
        type: int
      cold_storage_options:
        description:
          - Specifies the ColdStorageOptions config for a Domain.
        type: dict
        suboptions:
          enabled:
            description:
              - True to enable cold storage. Supported on Elasticsearch 7.9 or above.
            required: false
            type: bool
  ebs_options:
    description:
      - Parameters to configure EBS-based storage for an OpenSearch Service domain.
    type: dict
    suboptions:
      ebs_enabled:
        description:
          - Specifies whether EBS-based storage is enabled.
        required: false
        type: bool
      volume_type:
        description:
          - Specifies the volume type for EBS-based storage. "standard"|"gp2"|"io1"
        required: false
        type: str
      volume_size:
        description:
          - Integer to specify the size of an EBS volume.
        required: false
        type: int
      iops:
        description:
          - The IOPD for a Provisioned IOPS EBS volume (SSD).
        required: false
        type: int
  vpc_options:
    description:
      - Options to specify the subnets and security groups for a VPC endpoint.
    type: dict
    suboptions:
      subnets:
        description:
          - Specifies the subnet ids for VPC endpoint.
        required: false
        type: list
        elements: str
      security_groups:
        description:
          - Specifies the security group ids for VPC endpoint.
        required: false
        type: list
        elements: str
  snapshot_options:
    description:
      - Option to set time, in UTC format, of the daily automated snapshot.
    type: dict
    suboptions:
      automated_snapshot_start_hour:
        description:
          - >
            Integer value from 0 to 23 specifying when the service takes a daily automated snapshot
            of the specified Elasticsearch domain.
        required: false
        type: int
  access_policies:
    description:
      - IAM access policy as a JSON-formatted string.
    required: false
    type: dict
  encryption_at_rest_options:
    description:
      - Parameters to enable encryption at rest.
    type: dict
    suboptions:
      enabled:
        description:
          - Should data be encrypted while at rest.
        required: false
        type: bool
      kms_key_id:
        description:
          - If encryption at rest enabled, this identifies the encryption key to use.
          - The value should be a KMS key ARN. It can also be the KMS key id.
        required: false
        type: str
  node_to_node_encryption_options:
    description:
      - Node-to-node encryption options.
    type: dict
    suboptions:
      enabled:
        description:
            - True to enable node-to-node encryption.
        required: false
        type: bool
  cognito_options:
    description:
      - Parameters to configure OpenSearch Service to use Amazon Cognito authentication for OpenSearch Dashboards.
    type: dict
    suboptions:
      enabled:
        description:
          - The option to enable Cognito for OpenSearch Dashboards authentication.
        required: false
        type: bool
      user_pool_id:
        description:
          - The Cognito user pool ID for OpenSearch Dashboards authentication.
        required: false
        type: str
      identity_pool_id:
        description:
          - The Cognito identity pool ID for OpenSearch Dashboards authentication.
        required: false
        type: str
      role_arn:
        description:
          - The role ARN that provides OpenSearch permissions for accessing Cognito resources.
        required: false
        type: str
  domain_endpoint_options:
    description:
      - Options to specify configuration that will be applied to the domain endpoint.
    type: dict
    suboptions:
      enforce_https:
        description:
          - Whether only HTTPS endpoint should be enabled for the domain.
        type: bool
      tls_security_policy:
        description:
          - Specify the TLS security policy to apply to the HTTPS endpoint of the domain.
        type: str
      custom_endpoint_enabled:
        description:
          - Whether to enable a custom endpoint for the domain.
        type: bool
      custom_endpoint:
        description:
          - The fully qualified domain for your custom endpoint.
        type: str
      custom_endpoint_certificate_arn:
        description:
          - The ACM certificate ARN for your custom endpoint.
        type: str
  advanced_security_options:
    description:
      - Specifies advanced security options.
    type: dict
    suboptions:
      enabled:
        description:
          - True if advanced security is enabled.
          - You must enable node-to-node encryption to use advanced security options.
        type: bool
      internal_user_database_enabled:
        description:
          - True if the internal user database is enabled.
        type: bool
      master_user_options:
        description:
          - Credentials for the master user, username and password, ARN, or both.
        type: dict
        suboptions:
          master_user_arn:
            description:
              - ARN for the master user (if IAM is enabled).
            type: str
          master_user_name:
            description:
              - The username of the master user, which is stored in the Amazon OpenSearch Service domain internal database.
            type: str
          master_user_password:
            description:
              - The password of the master user, which is stored in the Amazon OpenSearch Service domain internal database.
            type: str
      saml_options:
        description:
          - The SAML application configuration for the domain.
        type: dict
        suboptions:
          enabled:
            description:
              - True if SAML is enabled.
              - To use SAML authentication, you must enable fine-grained access control.
              - You can only enable SAML authentication for OpenSearch Dashboards on existing domains,
                not during the creation of new ones.
              - Domains only support one Dashboards authentication method at a time.
                If you have Amazon Cognito authentication for OpenSearch Dashboards enabled,
                you must disable it before you can enable SAML.
            type: bool
          idp:
            description:
              - The SAML Identity Provider's information.
            type: dict
            suboptions:
              metadata_content:
                description:
                  - The metadata of the SAML application in XML format.
                type: str
              entity_id:
                description:
                  - The unique entity ID of the application in SAML identity provider.
                type: str
          master_user_name:
            description:
              - The SAML master username, which is stored in the Amazon OpenSearch Service domain internal database.
            type: str
          master_backend_role:
            description:
              - The backend role that the SAML master user is mapped to.
            type: str
          subject_key:
            description:
              - Element of the SAML assertion to use for username. Default is NameID.
            type: str
          roles_key:
            description:
              - Element of the SAML assertion to use for backend roles. Default is roles.
            type: str
          session_timeout_minutes:
            description:
              - The duration, in minutes, after which a user session becomes inactive. Acceptable values are between 1 and 1440, and the default value is 60.
            type: int
  auto_tune_options:
    description:
      - Specifies Auto-Tune options.
    type: dict
    suboptions:
      desired_state:
        description:
          - The Auto-Tune desired state. Valid values are ENABLED and DISABLED.
        type: str
        choices: ['ENABLED', 'DISABLED']
      maintenance_schedules:
        description:
          - A list of maintenance schedules.
        type: list
        elements: dict
        suboptions:
          start_at:
            description:
              - The timestamp at which the Auto-Tune maintenance schedule starts.
            type: str
          duration:
            description:
              - Specifies maintenance schedule duration, duration value and duration unit.
            type: dict
            suboptions:
              value:
                description:
                  - Integer to specify the value of a maintenance schedule duration.
                type: int
              unit:
                description:
                  - The unit of a maintenance schedule duration. Valid value is HOURS.
                choices: ['HOURS']
                type: str
          cron_expression_for_recurrence:
            description:
              - A cron expression for a recurring maintenance schedule.
            type: str
  wait:
    description:
      - Whether or not to wait for completion of OpenSearch creation, modification or deletion.
    type: bool
    default: false
  wait_timeout:
    description:
      - how long before wait gives up, in seconds.
    default: 300
    type: int
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.tags
  - amazon.aws.boto3
"""

RETURN = r""" # """

EXAMPLES = r"""

- name: Create OpenSearch domain for dev environment, no zone awareness, no dedicated masters
  community.aws.opensearch:
    domain_name: "dev-cluster"
    engine_version: Elasticsearch_1.1
    cluster_config:
      instance_type: "t2.small.search"
      instance_count: 2
      zone_awareness: false
      dedicated_master: false
    ebs_options:
      ebs_enabled: true
      volume_type: "gp2"
      volume_size: 10
    access_policies: "{{ lookup('file', 'policy.json') | from_json }}"

- name: Create OpenSearch domain with dedicated masters
  community.aws.opensearch:
    domain_name: "my-domain"
    engine_version: OpenSearch_1.1
    cluster_config:
      instance_type: "t2.small.search"
      instance_count: 12
      dedicated_master: true
      zone_awareness: true
      availability_zone_count: 2
      dedicated_master_instance_type: "t2.small.search"
      dedicated_master_instance_count: 3
      warm_enabled: true
      warm_type: "ultrawarm1.medium.search"
      warm_count: 1
      cold_storage_options:
        enabled: false
    ebs_options:
      ebs_enabled: true
      volume_type: "io1"
      volume_size: 10
      iops: 1000
    vpc_options:
      subnets:
        - "subnet-e537d64a"
        - "subnet-e537d64b"
      security_groups:
        - "sg-dd2f13cb"
        - "sg-dd2f13cc"
    snapshot_options:
      automated_snapshot_start_hour: 13
    access_policies: "{{ lookup('file', 'policy.json') | from_json }}"
    encryption_at_rest_options:
      enabled: false
    node_to_node_encryption_options:
      enabled: false
    auto_tune_options:
      enabled: true
      maintenance_schedules:
        - start_at: "2025-01-12"
          duration:
            value: 1
            unit: "HOURS"
          cron_expression_for_recurrence: "cron(0 12 * * ? *)"
        - start_at: "2032-01-12"
          duration:
            value: 2
            unit: "HOURS"
          cron_expression_for_recurrence: "cron(0 12 * * ? *)"
    tags:
      Environment: Development
      Application: Search
    wait: true

- name: Increase size of EBS volumes for existing cluster
  community.aws.opensearch:
    domain_name: "my-domain"
    ebs_options:
      volume_size: 5
    wait: true

- name: Increase instance count for existing cluster
  community.aws.opensearch:
    domain_name: "my-domain"
    cluster_config:
      instance_count: 40
    wait: true
"""

import datetime
import json
from copy import deepcopy

try:
    import botocore
except ImportError:
    pass  # handled by AnsibleAWSModule


from ansible_collections.amazon.aws.plugins.module_utils.botocore import is_boto3_error_code
from ansible_collections.amazon.aws.plugins.module_utils.policy import compare_policies
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule
from ansible_collections.community.aws.plugins.module_utils.opensearch import compare_domain_versions
from ansible_collections.community.aws.plugins.module_utils.opensearch import ensure_tags
from ansible_collections.community.aws.plugins.module_utils.opensearch import get_domain_config
from ansible_collections.community.aws.plugins.module_utils.opensearch import get_domain_status
from ansible_collections.community.aws.plugins.module_utils.opensearch import get_target_increment_version
from ansible_collections.community.aws.plugins.module_utils.opensearch import normalize_opensearch
from ansible_collections.community.aws.plugins.module_utils.opensearch import parse_version
from ansible_collections.community.aws.plugins.module_utils.opensearch import wait_for_domain_status


def ensure_domain_absent(client, module):
    domain_name = module.params.get("domain_name")
    changed = False

    domain = get_domain_status(client, module, domain_name)
    if module.check_mode:
        module.exit_json(changed=True, msg="Would have deleted domain if not in check mode")
    try:
        client.delete_domain(DomainName=domain_name)
        changed = True
    except is_boto3_error_code("ResourceNotFoundException"):
        # The resource does not exist, or it has already been deleted
        return dict(changed=False)
    except (
        botocore.exceptions.ClientError,
        botocore.exceptions.BotoCoreError,
    ) as e:  # pylint: disable=duplicate-except
        module.fail_json_aws(e, msg="trying to delete domain")

    # If we're not waiting for a delete to complete then we're all done
    # so just return
    if not domain or not module.params.get("wait"):
        return dict(changed=changed)
    try:
        wait_for_domain_status(client, module, domain_name, "domain_deleted")
        return dict(changed=changed)
    except is_boto3_error_code("ResourceNotFoundException"):
        return dict(changed=changed)
    except (
        botocore.exceptions.ClientError,
        botocore.exceptions.BotoCoreError,
    ) as e:  # pylint: disable=duplicate-except
        module.fail_json_aws(e, "awaiting domain deletion")


def upgrade_domain(client, module, source_version, target_engine_version):
    domain_name = module.params.get("domain_name")
    # Determine if it's possible to upgrade directly from source version
    # to target version, or if it's necessary to upgrade through intermediate major versions.
    next_version = target_engine_version
    # When perform_check_only is true, indicates that an upgrade eligibility check needs
    # to be performed. Does not actually perform the upgrade.
    perform_check_only = False
    if module.check_mode:
        perform_check_only = True
    current_version = source_version
    while current_version != target_engine_version:
        v = get_target_increment_version(client, module, domain_name, target_engine_version)
        if v is None:
            # There is no compatible version, according to the get_compatible_versions() API.
            # The upgrade should fail, but try anyway.
            next_version = target_engine_version
        if next_version != target_engine_version:
            # It's not possible to upgrade directly to the target version.
            # Check the module parameters to determine if this is allowed or not.
            if not module.params.get("allow_intermediate_upgrades"):
                module.fail_json(
                    msg=f"Cannot upgrade from {source_version} to version {target_engine_version}. The highest compatible version is {next_version}"
                )

        parameters = {
            "DomainName": domain_name,
            "TargetVersion": next_version,
            "PerformCheckOnly": perform_check_only,
        }

        if not module.check_mode:
            # If background tasks are in progress, wait until they complete.
            # This can take several hours depending on the cluster size and the type of background tasks
            # (maybe an upgrade is already in progress).
            # It's not possible to upgrade a domain that has background tasks are in progress,
            # the call to client.upgrade_domain would fail.
            wait_for_domain_status(client, module, domain_name, "domain_available")

        try:
            client.upgrade_domain(**parameters)
        except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
            # In check mode (=> PerformCheckOnly==True), a ValidationException may be
            # raised if it's not possible to upgrade to the target version.
            module.fail_json_aws(
                e,
                msg=f"Couldn't upgrade domain {domain_name} from {current_version} to {next_version}",
            )

        if module.check_mode:
            module.exit_json(
                changed=True,
                msg=f"Would have upgraded domain from {current_version} to {next_version} if not in check mode",
            )
        current_version = next_version

    if module.params.get("wait"):
        wait_for_domain_status(client, module, domain_name, "domain_available")


def set_cluster_config(module, current_domain_config, desired_domain_config, change_set):
    changed = False

    cluster_config = desired_domain_config["ClusterConfig"]
    cluster_opts = module.params.get("cluster_config")
    if cluster_opts is not None:
        if cluster_opts.get("instance_type") is not None:
            cluster_config["InstanceType"] = cluster_opts.get("instance_type")
        if cluster_opts.get("instance_count") is not None:
            cluster_config["InstanceCount"] = cluster_opts.get("instance_count")
        if cluster_opts.get("zone_awareness") is not None:
            cluster_config["ZoneAwarenessEnabled"] = cluster_opts.get("zone_awareness")
        if cluster_config["ZoneAwarenessEnabled"]:
            if cluster_opts.get("availability_zone_count") is not None:
                cluster_config["ZoneAwarenessConfig"] = {
                    "AvailabilityZoneCount": cluster_opts.get("availability_zone_count"),
                }

        if cluster_opts.get("dedicated_master") is not None:
            cluster_config["DedicatedMasterEnabled"] = cluster_opts.get("dedicated_master")
        if cluster_config["DedicatedMasterEnabled"]:
            if cluster_opts.get("dedicated_master_instance_type") is not None:
                cluster_config["DedicatedMasterType"] = cluster_opts.get("dedicated_master_instance_type")
            if cluster_opts.get("dedicated_master_instance_count") is not None:
                cluster_config["DedicatedMasterCount"] = cluster_opts.get("dedicated_master_instance_count")

        if cluster_opts.get("warm_enabled") is not None:
            cluster_config["WarmEnabled"] = cluster_opts.get("warm_enabled")
        if cluster_config["WarmEnabled"]:
            if cluster_opts.get("warm_type") is not None:
                cluster_config["WarmType"] = cluster_opts.get("warm_type")
            if cluster_opts.get("warm_count") is not None:
                cluster_config["WarmCount"] = cluster_opts.get("warm_count")

    cold_storage_opts = None
    if cluster_opts is not None:
        cold_storage_opts = cluster_opts.get("cold_storage_options")
    if compare_domain_versions(desired_domain_config["EngineVersion"], "Elasticsearch_7.9") < 0:
        # If the engine version is ElasticSearch < 7.9, cold storage is not supported.
        # When querying a domain < 7.9, the AWS API indicates cold storage is disabled (Enabled: False),
        # which makes sense. However, trying to do HTTP POST with Enable: False causes an API error.
        # The 'ColdStorageOptions' attribute should not be present in HTTP POST.
        if cold_storage_opts is not None and cold_storage_opts.get("enabled"):
            module.fail_json(msg="Cold Storage is not supported")
        cluster_config.pop("ColdStorageOptions", None)
        if current_domain_config is not None and "ClusterConfig" in current_domain_config:
            # Remove 'ColdStorageOptions' from the current domain config, otherwise the actual vs desired diff
            # will indicate a change must be done.
            current_domain_config["ClusterConfig"].pop("ColdStorageOptions", None)
    else:
        # Elasticsearch 7.9 and above support ColdStorageOptions.
        if cold_storage_opts is not None and cold_storage_opts.get("enabled") is not None:
            cluster_config["ColdStorageOptions"] = {
                "Enabled": cold_storage_opts.get("enabled"),
            }

    if current_domain_config is not None and current_domain_config["ClusterConfig"] != cluster_config:
        change_set.append(f"ClusterConfig changed from {current_domain_config['ClusterConfig']} to {cluster_config}")
        changed = True
    return changed


def set_ebs_options(module, current_domain_config, desired_domain_config, change_set):
    changed = False
    ebs_config = desired_domain_config["EBSOptions"]
    ebs_opts = module.params.get("ebs_options")
    if ebs_opts is None:
        return changed
    if ebs_opts.get("ebs_enabled") is not None:
        ebs_config["EBSEnabled"] = ebs_opts.get("ebs_enabled")

    if not ebs_config["EBSEnabled"]:
        desired_domain_config["EBSOptions"] = {
            "EBSEnabled": False,
        }
    else:
        if ebs_opts.get("volume_type") is not None:
            ebs_config["VolumeType"] = ebs_opts.get("volume_type")
        if ebs_opts.get("volume_size") is not None:
            ebs_config["VolumeSize"] = ebs_opts.get("volume_size")
        if ebs_opts.get("iops") is not None:
            ebs_config["Iops"] = ebs_opts.get("iops")

    if current_domain_config is not None and current_domain_config["EBSOptions"] != ebs_config:
        change_set.append(f"EBSOptions changed from {current_domain_config['EBSOptions']} to {ebs_config}")
        changed = True
    return changed


def set_encryption_at_rest_options(module, current_domain_config, desired_domain_config, change_set):
    changed = False
    encryption_at_rest_config = desired_domain_config["EncryptionAtRestOptions"]
    encryption_at_rest_opts = module.params.get("encryption_at_rest_options")
    if encryption_at_rest_opts is None:
        return False
    if encryption_at_rest_opts.get("enabled") is not None:
        encryption_at_rest_config["Enabled"] = encryption_at_rest_opts.get("enabled")
    if not encryption_at_rest_config["Enabled"]:
        desired_domain_config["EncryptionAtRestOptions"] = {
            "Enabled": False,
        }
    else:
        if encryption_at_rest_opts.get("kms_key_id") is not None:
            encryption_at_rest_config["KmsKeyId"] = encryption_at_rest_opts.get("kms_key_id")

    if (
        current_domain_config is not None
        and current_domain_config["EncryptionAtRestOptions"] != encryption_at_rest_config
    ):
        change_set.append(
            f"EncryptionAtRestOptions changed from {current_domain_config['EncryptionAtRestOptions']} to"
            f" {encryption_at_rest_config}"
        )
        changed = True
    return changed


def set_node_to_node_encryption_options(module, current_domain_config, desired_domain_config, change_set):
    changed = False
    node_to_node_encryption_config = desired_domain_config["NodeToNodeEncryptionOptions"]
    node_to_node_encryption_opts = module.params.get("node_to_node_encryption_options")
    if node_to_node_encryption_opts is None:
        return changed
    if node_to_node_encryption_opts.get("enabled") is not None:
        node_to_node_encryption_config["Enabled"] = node_to_node_encryption_opts.get("enabled")

    if (
        current_domain_config is not None
        and current_domain_config["NodeToNodeEncryptionOptions"] != node_to_node_encryption_config
    ):
        change_set.append(
            f"NodeToNodeEncryptionOptions changed from {current_domain_config['NodeToNodeEncryptionOptions']} to"
            f" {node_to_node_encryption_config}"
        )
        changed = True
    return changed


def set_vpc_options(module, current_domain_config, desired_domain_config, change_set):
    changed = False
    vpc_config = None
    if "VPCOptions" in desired_domain_config:
        vpc_config = desired_domain_config["VPCOptions"]
    vpc_opts = module.params.get("vpc_options")
    if vpc_opts is None:
        return changed
    vpc_subnets = vpc_opts.get("subnets")
    if vpc_subnets is not None:
        if vpc_config is None:
            vpc_config = {}
            desired_domain_config["VPCOptions"] = vpc_config
        # OpenSearch cluster is attached to VPC
        if isinstance(vpc_subnets, str):
            vpc_subnets = [x.strip() for x in vpc_subnets.split(",")]
        vpc_config["SubnetIds"] = vpc_subnets

    vpc_security_groups = vpc_opts.get("security_groups")
    if vpc_security_groups is not None:
        if vpc_config is None:
            vpc_config = {}
            desired_domain_config["VPCOptions"] = vpc_config
        if isinstance(vpc_security_groups, str):
            vpc_security_groups = [x.strip() for x in vpc_security_groups.split(",")]
        vpc_config["SecurityGroupIds"] = vpc_security_groups

    if current_domain_config is not None:
        # Modify existing cluster.
        current_cluster_is_vpc = False
        desired_cluster_is_vpc = False
        if (
            "VPCOptions" in current_domain_config
            and "SubnetIds" in current_domain_config["VPCOptions"]
            and len(current_domain_config["VPCOptions"]["SubnetIds"]) > 0
        ):
            current_cluster_is_vpc = True
        if (
            "VPCOptions" in desired_domain_config
            and "SubnetIds" in desired_domain_config["VPCOptions"]
            and len(desired_domain_config["VPCOptions"]["SubnetIds"]) > 0
        ):
            desired_cluster_is_vpc = True
        if current_cluster_is_vpc != desired_cluster_is_vpc:
            # AWS does not allow changing the type. Don't fail here so we return the AWS API error.
            change_set.append("VPCOptions changed between Internet and VPC")
            changed = True
        elif desired_cluster_is_vpc is False:
            # There are no VPCOptions to configure.
            pass
        else:
            # Note the subnets may be the same but be listed in a different order.
            if set(current_domain_config["VPCOptions"]["SubnetIds"]) != set(vpc_config["SubnetIds"]):
                change_set.append(
                    f"SubnetIds changed from {current_domain_config['VPCOptions']['SubnetIds']} to"
                    f" {vpc_config['SubnetIds']}"
                )
                changed = True
            if set(current_domain_config["VPCOptions"]["SecurityGroupIds"]) != set(vpc_config["SecurityGroupIds"]):
                change_set.append(
                    f"SecurityGroup changed from {current_domain_config['VPCOptions']['SecurityGroupIds']} to"
                    f" {vpc_config['SecurityGroupIds']}"
                )
                changed = True
    return changed


def set_snapshot_options(module, current_domain_config, desired_domain_config, change_set):
    changed = False
    snapshot_config = desired_domain_config["SnapshotOptions"]
    snapshot_opts = module.params.get("snapshot_options")
    if snapshot_opts is None:
        return changed
    if snapshot_opts.get("automated_snapshot_start_hour") is not None:
        snapshot_config["AutomatedSnapshotStartHour"] = snapshot_opts.get("automated_snapshot_start_hour")
    if current_domain_config is not None and current_domain_config["SnapshotOptions"] != snapshot_config:
        change_set.append("SnapshotOptions changed")
        changed = True
    return changed


def set_cognito_options(module, current_domain_config, desired_domain_config, change_set):
    changed = False
    cognito_config = desired_domain_config["CognitoOptions"]
    cognito_opts = module.params.get("cognito_options")
    if cognito_opts is None:
        return changed
    if cognito_opts.get("enabled") is not None:
        cognito_config["Enabled"] = cognito_opts.get("enabled")
    if not cognito_config["Enabled"]:
        desired_domain_config["CognitoOptions"] = {
            "Enabled": False,
        }
    else:
        if cognito_opts.get("cognito_user_pool_id") is not None:
            cognito_config["UserPoolId"] = cognito_opts.get("cognito_user_pool_id")
        if cognito_opts.get("cognito_identity_pool_id") is not None:
            cognito_config["IdentityPoolId"] = cognito_opts.get("cognito_identity_pool_id")
        if cognito_opts.get("cognito_role_arn") is not None:
            cognito_config["RoleArn"] = cognito_opts.get("cognito_role_arn")

    if current_domain_config is not None and current_domain_config["CognitoOptions"] != cognito_config:
        change_set.append(f"CognitoOptions changed from {current_domain_config['CognitoOptions']} to {cognito_config}")
        changed = True
    return changed


def set_advanced_security_options(module, current_domain_config, desired_domain_config, change_set):
    changed = False
    advanced_security_config = desired_domain_config["AdvancedSecurityOptions"]
    advanced_security_opts = module.params.get("advanced_security_options")
    if advanced_security_opts is None:
        return changed
    if advanced_security_opts.get("enabled") is not None:
        advanced_security_config["Enabled"] = advanced_security_opts.get("enabled")
    if not advanced_security_config["Enabled"]:
        desired_domain_config["AdvancedSecurityOptions"] = {
            "Enabled": False,
        }
    else:
        if advanced_security_opts.get("internal_user_database_enabled") is not None:
            advanced_security_config["InternalUserDatabaseEnabled"] = advanced_security_opts.get(
                "internal_user_database_enabled"
            )
        master_user_opts = advanced_security_opts.get("master_user_options")
        if master_user_opts is not None:
            advanced_security_config.setdefault("MasterUserOptions", {})
            if master_user_opts.get("master_user_arn") is not None:
                advanced_security_config["MasterUserOptions"]["MasterUserARN"] = master_user_opts.get("master_user_arn")
            if master_user_opts.get("master_user_name") is not None:
                advanced_security_config["MasterUserOptions"]["MasterUserName"] = master_user_opts.get(
                    "master_user_name"
                )
            if master_user_opts.get("master_user_password") is not None:
                advanced_security_config["MasterUserOptions"]["MasterUserPassword"] = master_user_opts.get(
                    "master_user_password"
                )
        saml_opts = advanced_security_opts.get("saml_options")
        if saml_opts is not None:
            if saml_opts.get("enabled") is not None:
                advanced_security_config["SamlOptions"]["Enabled"] = saml_opts.get("enabled")
            idp_opts = saml_opts.get("idp")
            if idp_opts is not None:
                if idp_opts.get("metadata_content") is not None:
                    advanced_security_config["SamlOptions"]["Idp"]["MetadataContent"] = idp_opts.get("metadata_content")
                if idp_opts.get("entity_id") is not None:
                    advanced_security_config["SamlOptions"]["Idp"]["EntityId"] = idp_opts.get("entity_id")
            if saml_opts.get("master_user_name") is not None:
                advanced_security_config["SamlOptions"]["MasterUserName"] = saml_opts.get("master_user_name")
            if saml_opts.get("master_backend_role") is not None:
                advanced_security_config["SamlOptions"]["MasterBackendRole"] = saml_opts.get("master_backend_role")
            if saml_opts.get("subject_key") is not None:
                advanced_security_config["SamlOptions"]["SubjectKey"] = saml_opts.get("subject_key")
            if saml_opts.get("roles_key") is not None:
                advanced_security_config["SamlOptions"]["RolesKey"] = saml_opts.get("roles_key")
            if saml_opts.get("session_timeout_minutes") is not None:
                advanced_security_config["SamlOptions"]["SessionTimeoutMinutes"] = saml_opts.get(
                    "session_timeout_minutes"
                )

    if (
        current_domain_config is not None
        and current_domain_config["AdvancedSecurityOptions"] != advanced_security_config
    ):
        change_set.append(
            f"AdvancedSecurityOptions changed from {current_domain_config['AdvancedSecurityOptions']} to"
            f" {advanced_security_config}"
        )
        changed = True
    return changed


def set_domain_endpoint_options(module, current_domain_config, desired_domain_config, change_set):
    changed = False
    domain_endpoint_config = desired_domain_config["DomainEndpointOptions"]
    domain_endpoint_opts = module.params.get("domain_endpoint_options")
    if domain_endpoint_opts is None:
        return changed
    if domain_endpoint_opts.get("enforce_https") is not None:
        domain_endpoint_config["EnforceHTTPS"] = domain_endpoint_opts.get("enforce_https")
    if domain_endpoint_opts.get("tls_security_policy") is not None:
        domain_endpoint_config["TLSSecurityPolicy"] = domain_endpoint_opts.get("tls_security_policy")
    if domain_endpoint_opts.get("custom_endpoint_enabled") is not None:
        domain_endpoint_config["CustomEndpointEnabled"] = domain_endpoint_opts.get("custom_endpoint_enabled")
    if domain_endpoint_config["CustomEndpointEnabled"]:
        if domain_endpoint_opts.get("custom_endpoint") is not None:
            domain_endpoint_config["CustomEndpoint"] = domain_endpoint_opts.get("custom_endpoint")
        if domain_endpoint_opts.get("custom_endpoint_certificate_arn") is not None:
            domain_endpoint_config["CustomEndpointCertificateArn"] = domain_endpoint_opts.get(
                "custom_endpoint_certificate_arn"
            )

    if current_domain_config is not None and current_domain_config["DomainEndpointOptions"] != domain_endpoint_config:
        change_set.append(
            f"DomainEndpointOptions changed from {current_domain_config['DomainEndpointOptions']} to"
            f" {domain_endpoint_config}"
        )
        changed = True
    return changed


def set_auto_tune_options(module, current_domain_config, desired_domain_config, change_set):
    changed = False
    auto_tune_config = desired_domain_config["AutoTuneOptions"]
    auto_tune_opts = module.params.get("auto_tune_options")
    if auto_tune_opts is None:
        return changed
    schedules = auto_tune_opts.get("maintenance_schedules")
    if auto_tune_opts.get("desired_state") is not None:
        auto_tune_config["DesiredState"] = auto_tune_opts.get("desired_state")
    if auto_tune_config["DesiredState"] != "ENABLED":
        desired_domain_config["AutoTuneOptions"] = {
            "DesiredState": "DISABLED",
        }
    elif schedules is not None:
        auto_tune_config["MaintenanceSchedules"] = []
        for s in schedules:
            schedule_entry = {}
            start_at = s.get("start_at")
            if start_at is not None:
                if isinstance(start_at, datetime.datetime):
                    # The property was parsed from yaml to datetime, but the AWS API wants a string
                    start_at = start_at.strftime("%Y-%m-%d")
                schedule_entry["StartAt"] = start_at
            duration_opt = s.get("duration")
            if duration_opt is not None:
                schedule_entry["Duration"] = {}
                if duration_opt.get("value") is not None:
                    schedule_entry["Duration"]["Value"] = duration_opt.get("value")
                if duration_opt.get("unit") is not None:
                    schedule_entry["Duration"]["Unit"] = duration_opt.get("unit")
            if s.get("cron_expression_for_recurrence") is not None:
                schedule_entry["CronExpressionForRecurrence"] = s.get("cron_expression_for_recurrence")
            auto_tune_config["MaintenanceSchedules"].append(schedule_entry)
    if current_domain_config is not None:
        if current_domain_config["AutoTuneOptions"]["DesiredState"] != auto_tune_config["DesiredState"]:
            change_set.append(
                "AutoTuneOptions.DesiredState changed from"
                f" {current_domain_config['AutoTuneOptions']['DesiredState']} to {auto_tune_config['DesiredState']}"
            )
            changed = True
        if auto_tune_config["MaintenanceSchedules"] != current_domain_config["AutoTuneOptions"]["MaintenanceSchedules"]:
            change_set.append(
                "AutoTuneOptions.MaintenanceSchedules changed from"
                f" {current_domain_config['AutoTuneOptions']['MaintenanceSchedules']} to"
                f" {auto_tune_config['MaintenanceSchedules']}"
            )
            changed = True
    return changed


def set_access_policy(module, current_domain_config, desired_domain_config, change_set):
    access_policy_config = None
    changed = False
    access_policy_opt = module.params.get("access_policies")
    if access_policy_opt is None:
        return changed
    try:
        access_policy_config = json.dumps(access_policy_opt)
    except Exception as e:
        module.fail_json(msg=f"Failed to convert the policy into valid JSON: {str(e)}")
    if current_domain_config is not None:
        # Updating existing domain
        current_access_policy = json.loads(current_domain_config["AccessPolicies"])
        if not compare_policies(current_access_policy, access_policy_opt):
            change_set.append(f"AccessPolicy changed from {current_access_policy} to {access_policy_opt}")
            changed = True
            desired_domain_config["AccessPolicies"] = access_policy_config
    else:
        # Creating new domain
        desired_domain_config["AccessPolicies"] = access_policy_config
    return changed


def ensure_domain_present(client, module):
    domain_name = module.params.get("domain_name")

    # Create default if OpenSearch does not exist. If domain already exists,
    # the data is populated by retrieving the current configuration from the API.
    desired_domain_config = {
        "DomainName": module.params.get("domain_name"),
        "EngineVersion": "OpenSearch_1.1",
        "ClusterConfig": {
            "InstanceType": "t2.small.search",
            "InstanceCount": 2,
            "ZoneAwarenessEnabled": False,
            "DedicatedMasterEnabled": False,
            "WarmEnabled": False,
        },
        # By default create ES attached to the Internet.
        # If the "VPCOptions" property is specified, even if empty, the API server interprets
        # as incomplete VPC configuration.
        # "VPCOptions": {},
        "EBSOptions": {
            "EBSEnabled": False,
        },
        "EncryptionAtRestOptions": {
            "Enabled": False,
        },
        "NodeToNodeEncryptionOptions": {
            "Enabled": False,
        },
        "SnapshotOptions": {
            "AutomatedSnapshotStartHour": 0,
        },
        "CognitoOptions": {
            "Enabled": False,
        },
        "AdvancedSecurityOptions": {
            "Enabled": False,
        },
        "DomainEndpointOptions": {
            "CustomEndpointEnabled": False,
        },
        "AutoTuneOptions": {
            "DesiredState": "DISABLED",
        },
    }
    # Determine if OpenSearch domain already exists.
    # current_domain_config may be None if the domain does not exist.
    (current_domain_config, domain_arn) = get_domain_config(client, module, domain_name)
    if current_domain_config is not None:
        desired_domain_config = deepcopy(current_domain_config)

    if module.params.get("engine_version") is not None:
        # Validate the engine_version
        v = parse_version(module.params.get("engine_version"))
        if v is None:
            module.fail_json("Invalid engine_version. Must be Elasticsearch_X.Y or OpenSearch_X.Y")
        desired_domain_config["EngineVersion"] = module.params.get("engine_version")

    changed = False
    change_set = []  # For check mode purpose

    changed |= set_cluster_config(module, current_domain_config, desired_domain_config, change_set)
    changed |= set_ebs_options(module, current_domain_config, desired_domain_config, change_set)
    changed |= set_encryption_at_rest_options(module, current_domain_config, desired_domain_config, change_set)
    changed |= set_node_to_node_encryption_options(module, current_domain_config, desired_domain_config, change_set)
    changed |= set_vpc_options(module, current_domain_config, desired_domain_config, change_set)
    changed |= set_snapshot_options(module, current_domain_config, desired_domain_config, change_set)
    changed |= set_cognito_options(module, current_domain_config, desired_domain_config, change_set)
    changed |= set_advanced_security_options(module, current_domain_config, desired_domain_config, change_set)
    changed |= set_domain_endpoint_options(module, current_domain_config, desired_domain_config, change_set)
    changed |= set_auto_tune_options(module, current_domain_config, desired_domain_config, change_set)
    changed |= set_access_policy(module, current_domain_config, desired_domain_config, change_set)

    if current_domain_config is not None:
        if desired_domain_config["EngineVersion"] != current_domain_config["EngineVersion"]:
            changed = True
            change_set.append("EngineVersion changed")
            upgrade_domain(
                client,
                module,
                current_domain_config["EngineVersion"],
                desired_domain_config["EngineVersion"],
            )

        if changed:
            if module.check_mode:
                module.exit_json(
                    changed=True,
                    msg=f"Would have updated domain if not in check mode: {change_set}",
                )
            # Remove the "EngineVersion" attribute, the AWS API does not accept this attribute.
            desired_domain_config.pop("EngineVersion", None)
            try:
                client.update_domain_config(**desired_domain_config)
            except (
                botocore.exceptions.BotoCoreError,
                botocore.exceptions.ClientError,
            ) as e:
                module.fail_json_aws(e, msg=f"Couldn't update domain {domain_name}")

    else:
        # Create new OpenSearch cluster
        if module.params.get("access_policies") is None:
            module.fail_json("state is present but the following is missing: access_policies")

        changed = True
        if module.check_mode:
            module.exit_json(changed=True, msg="Would have created a domain if not in check mode")
        try:
            response = client.create_domain(**desired_domain_config)
            domain = response["DomainStatus"]
            domain_arn = domain["ARN"]
        except (
            botocore.exceptions.BotoCoreError,
            botocore.exceptions.ClientError,
        ) as e:
            module.fail_json_aws(e, msg=f"Couldn't update domain {domain_name}")

    try:
        existing_tags = boto3_tag_list_to_ansible_dict(client.list_tags(ARN=domain_arn, aws_retry=True)["TagList"])
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, f"Couldn't get tags for domain {domain_name}")

    desired_tags = module.params["tags"]
    purge_tags = module.params["purge_tags"]
    changed |= ensure_tags(client, module, domain_arn, existing_tags, desired_tags, purge_tags)

    if module.params.get("wait") and not module.check_mode:
        wait_for_domain_status(client, module, domain_name, "domain_available")

    domain = get_domain_status(client, module, domain_name)

    return dict(changed=changed, **normalize_opensearch(client, module, domain))


def main():
    module = AnsibleAWSModule(
        argument_spec=dict(
            state=dict(choices=["present", "absent"], default="present"),
            domain_name=dict(required=True),
            engine_version=dict(),
            allow_intermediate_upgrades=dict(required=False, type="bool", default=True),
            access_policies=dict(required=False, type="dict"),
            cluster_config=dict(
                type="dict",
                default=None,
                options=dict(
                    instance_type=dict(),
                    instance_count=dict(required=False, type="int"),
                    zone_awareness=dict(required=False, type="bool"),
                    availability_zone_count=dict(required=False, type="int"),
                    dedicated_master=dict(required=False, type="bool"),
                    dedicated_master_instance_type=dict(),
                    dedicated_master_instance_count=dict(type="int"),
                    warm_enabled=dict(required=False, type="bool"),
                    warm_type=dict(required=False),
                    warm_count=dict(required=False, type="int"),
                    cold_storage_options=dict(
                        type="dict",
                        default=None,
                        options=dict(
                            enabled=dict(required=False, type="bool"),
                        ),
                    ),
                ),
            ),
            snapshot_options=dict(
                type="dict",
                default=None,
                options=dict(
                    automated_snapshot_start_hour=dict(required=False, type="int"),
                ),
            ),
            ebs_options=dict(
                type="dict",
                default=None,
                options=dict(
                    ebs_enabled=dict(required=False, type="bool"),
                    volume_type=dict(required=False),
                    volume_size=dict(required=False, type="int"),
                    iops=dict(required=False, type="int"),
                ),
            ),
            vpc_options=dict(
                type="dict",
                default=None,
                options=dict(
                    subnets=dict(type="list", elements="str", required=False),
                    security_groups=dict(type="list", elements="str", required=False),
                ),
            ),
            cognito_options=dict(
                type="dict",
                default=None,
                options=dict(
                    enabled=dict(required=False, type="bool"),
                    user_pool_id=dict(required=False),
                    identity_pool_id=dict(required=False),
                    role_arn=dict(required=False, no_log=False),
                ),
            ),
            encryption_at_rest_options=dict(
                type="dict",
                default=None,
                options=dict(
                    enabled=dict(type="bool"),
                    kms_key_id=dict(required=False),
                ),
            ),
            node_to_node_encryption_options=dict(
                type="dict",
                default=None,
                options=dict(
                    enabled=dict(type="bool"),
                ),
            ),
            domain_endpoint_options=dict(
                type="dict",
                default=None,
                options=dict(
                    enforce_https=dict(type="bool"),
                    tls_security_policy=dict(),
                    custom_endpoint_enabled=dict(type="bool"),
                    custom_endpoint=dict(),
                    custom_endpoint_certificate_arn=dict(),
                ),
            ),
            advanced_security_options=dict(
                type="dict",
                default=None,
                options=dict(
                    enabled=dict(type="bool"),
                    internal_user_database_enabled=dict(type="bool"),
                    master_user_options=dict(
                        type="dict",
                        default=None,
                        options=dict(
                            master_user_arn=dict(),
                            master_user_name=dict(),
                            master_user_password=dict(no_log=True),
                        ),
                    ),
                    saml_options=dict(
                        type="dict",
                        default=None,
                        options=dict(
                            enabled=dict(type="bool"),
                            idp=dict(
                                type="dict",
                                default=None,
                                options=dict(
                                    metadata_content=dict(),
                                    entity_id=dict(),
                                ),
                            ),
                            master_user_name=dict(),
                            master_backend_role=dict(),
                            subject_key=dict(no_log=False),
                            roles_key=dict(no_log=False),
                            session_timeout_minutes=dict(type="int"),
                        ),
                    ),
                ),
            ),
            auto_tune_options=dict(
                type="dict",
                default=None,
                options=dict(
                    desired_state=dict(choices=["ENABLED", "DISABLED"]),
                    maintenance_schedules=dict(
                        type="list",
                        elements="dict",
                        default=None,
                        options=dict(
                            start_at=dict(),
                            duration=dict(
                                type="dict",
                                default=None,
                                options=dict(
                                    value=dict(type="int"),
                                    unit=dict(choices=["HOURS"]),
                                ),
                            ),
                            cron_expression_for_recurrence=dict(),
                        ),
                    ),
                ),
            ),
            tags=dict(type="dict", aliases=["resource_tags"]),
            purge_tags=dict(type="bool", default=True),
            wait=dict(type="bool", default=False),
            wait_timeout=dict(type="int", default=300),
        ),
        supports_check_mode=True,
    )

    try:
        client = module.client("opensearch", retry_decorator=AWSRetry.jittered_backoff())
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to connect to AWS opensearch service")

    if module.params["state"] == "absent":
        ret_dict = ensure_domain_absent(client, module)
    else:
        ret_dict = ensure_domain_present(client, module)

    module.exit_json(**ret_dict)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2025, Akini Ross (@akinross) <akinross@cisco.com>

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
module: ndo_l3out_secondary_ip
version_added: "2.12.0"
short_description: Manage L3Out Secondary IP Address on Cisco Nexus Dashboard Orchestrator (NDO).
description:
- Manage L3Out L3Out Secondary IP Address on Cisco Nexus Dashboard Orchestrator (NDO).
- This module is only supported on ND v3.1 (NDO v4.3) and later.
author:
- Akini Ross (@akinross)
options:
  template:
    description:
    - The name of the L3Out template.
    - This parameter or O(template_id) is required.
    type: str
    aliases: [ l3out_template ]
  template_id:
    description:
    - The ID of the L3Out template.
    - This parameter or O(template) is required.
    type: str
    aliases: [ l3out_template_id ]
  l3out:
    description:
    - The name of the L3Out.
    - This parameter or O(l3out_uuid) is required.
    type: str
  l3out_uuid:
    description:
    - The UUID of the L3Out.
    - This parameter or O(l3out) is required.
    type: str
  parent_type:
    description:
    - The parent type of the L3Out.
    type: str
    required: true
    choices: [ floating_svi, routed, routed_sub, svi, floating_svi_path_attributes ]
  node_id:
    description:
    - The ID of the node (border leaf switch).
    type: str
    aliases: [ node, border_leaf, anchor_node ]
  path:
    description:
    - The path of the interface.
    - The path must be an existing physical port (eg. eth1/1).
    type: str
    aliases: [ interface ]
  port_channel:
    description:
    - The port channel details.
    type: dict
    aliases: [ pc ]
    suboptions:
      uuid:
        description:
        - The UUID of the port channel.
        - This parameter or O(port_channel.reference) is required.
        type: str
      reference:
        description:
        - The reference details of the port channel.
        - This parameter or O(port_channel.uuid) is required.
        type: dict
        aliases: [ ref ]
        suboptions:
          name:
            description:
            - The name of the port channel.
            type: str
            required: true
          template:
            description:
            - The name of the template that contains the port channel.
            - This parameter or O(port_channel.reference.template_id) is required.
            type: str
          template_id:
            description:
            - The ID of the template that contains the port channel.
            - This parameter or O(port_channel.reference.template) is required.
            type: str
  virtual_port_channel:
    description:
    - The virtual port channel details.
    type: dict
    aliases: [ vpc ]
    suboptions:
      uuid:
        description:
        - The UUID of the virtual port channel.
        - This parameter or O(virtual_port_channel.reference) is required.
        type: str
      reference:
        description:
        - The reference details of the virtual port channel.
        - This parameter or O(virtual_port_channel.uuid) is required.
        type: dict
        aliases: [ ref ]
        suboptions:
          name:
            description:
            - The name of the virtual port channel.
            type: str
            required: true
          template:
            description:
            - The name of the template that contains the virtual port channel.
            - This parameter or O(virtual_port_channel.reference.template_id) is required.
            type: str
          template_id:
            description:
            - The ID of the template that contains the virtual port channel.
            - This parameter or O(virtual_port_channel.reference.template) is required.
            type: str
      side_b:
        description:
        - Whether the side of the virtual port channel interface is side B.
        type: bool
        default: false
  encapsulation_type:
    description:
    - The encapsulation type of the interface.
    type: str
    aliases: [ encap_type ]
    choices: [ vlan, vxlan ]
  encapsulation_value:
    description:
    - The encapsulation value of the interface.
    - The option O(encapsulation_type=vlan), specifies VLAN ID which must be in the range 1 - 4094.
    - The option O(encapsulation_type=vxlan), specifies VXLAN Network Identifier (VNI) which must be in the range 5000 - 16777215.
    type: int
    aliases: [ encap, encapsulation, encapsulation_id ]
  domain_type:
    description:
    - The type of the domain.
    type: str
    choices: [ vmm, physical ]
  domain_provider:
    description:
    - The provider of the domain.
    type: str
    choices: [ cloudfoundry, kubernetes, microsoft, openshift, openstack, redhat, vmware, nutanix ]
  domain:
    description:
    - The name of the domain.
    type: str
    aliases: [ domain_name ]
  secondary_address:
    description:
    - The secondary IP address.
    type: str
    aliases: [ ipv4, ipv6, ip ]
  dhcp_relay:
    description:
    - Whether to enable Dynamic Host Configuration Protocol (DHCP) relay.
    - If this parameter is unspecified, NDO defaults to O(dhcp_relay=false).
    type: bool
  nd_ra_prefix:
    description:
    - Whether to enable Neighbor Discovery (ND) Router Advertisement (RA).
    - If this parameter is unspecified, NDO defaults to O(nd_ra_prefix=false).
    type: bool
  ipv6_dad:
    description:
    - Whether to enable IPv6 Duplicate Address Detection (DAD).
    - If this parameter is unspecified, NDO defaults to O(ipv6_dad=disabled) when O(parent_type=floating_svi_path_attributes).
    - If this parameter is unspecified, NDO defaults to O(ipv6_dad=enabled) for all other parent types.
    type: str
    choices: [ enabled, disabled ]
  state:
    description:
    - Use C(absent) for removing.
    - Use C(query) for listing an object or multiple objects.
    - Use C(present) for creating or updating.
    type: str
    choices: [ absent, query, present ]
    default: query
notes:
- The O(template) must exist before using this module in your playbook.
  Use M(cisco.mso.ndo_template) to create the L3Out template.
- The O(l3out) must exist before using this module in your playbook.
  Use M(cisco.mso.ndo_l3out_template) to create the L3Out.
- The parent object defined by O(parent_type) must exist before using this module in your playbook.
  Use M(cisco.mso.ndo_l3out_routed_interface) to create the L3Out Routed Interface.
  Use M(cisco.mso.ndo_l3out_routed_sub_interface) to create the L3Out Routed Sub-Interface.
  Use M(cisco.mso.ndo_l3out_svi_interface) to create the L3Out SVI Interface.
  Use M(cisco.mso.ndo_l3out_floating_svi_interface) to create the L3Out Floating SVI Interface.
  Use M(cisco.mso.ndo_l3out_floating_svi_interface_path_attributes) to create the L3Out Floating SVI Interface Path Attributes.

seealso:
- module: cisco.mso.ndo_template
- module: cisco.mso.ndo_l3out_template
- module: cisco.mso.ndo_l3out_routed_interface
- module: cisco.mso.ndo_l3out_routed_sub_interface
- module: cisco.mso.ndo_l3out_svi_interface
- module: cisco.mso.ndo_l3out_floating_svi_interface
- module: cisco.mso.ndo_l3out_floating_svi_interface_path_attributes
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Create a secondary address on a L3out Routed Interface of type port
  cisco.mso.ndo_l3out_secondary_ip:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: l3out_template
    l3out: l3out_1
    node_id: 101
    path: eth1/3
    parent_type: routed
    secondary_address: 3::102/16
    state: present

- name: Create a secondary address on a L3out Routed Interface of type port channel
  cisco.mso.ndo_l3out_secondary_ip:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: l3out_template
    l3out: l3out_1
    port_channel:
      uuid: '{{ port_channel_interface_3.current.uuid }}'
    parent_type: routed
    secondary_address: 10.2.0.2/24
    state: present

- name: Create a secondary address on a L3out SVI Interface of type port
  cisco.mso.ndo_l3out_secondary_ip:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: l3out_template
    l3out: l3out_1
    node_id: 101
    path: eth1/1
    encapsulation_type: vlan
    encapsulation_value: 103
    parent_type: svi
    secondary_address: 10.0.0.2/24
    state: present

- name: Create a secondary address on a L3out SVI Interface of type port channel
  cisco.mso.ndo_l3out_secondary_ip:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: l3out_template
    l3out: l3out_1
    port_channel:
      uuid: '{{ port_channel_interface_1.current.uuid }}'
    encapsulation_type: vxlan
    encapsulation_value: 50001
    parent_type: svi
    secondary_address: 1::102/16
    state: present

- name: Create a secondary address on a L3out SVI Interface of type virtual port channel side a
  cisco.mso.ndo_l3out_secondary_ip:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: l3out_template
    l3out: l3out_1
    virtual_port_channel:
      uuid: '{{ virtual_port_channel_interface_1.current.uuid }}'
    encapsulation_type: vlan
    encapsulation_value: 1000
    parent_type: svi
    secondary_address: 10.0.2.3/24
    state: present

- name: Create a secondary address on a L3out SVI Interface of type virtual port channel side b
  cisco.mso.ndo_l3out_secondary_ip:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: l3out_template
    l3out: l3out_1
    virtual_port_channel:
      uuid: '{{ virtual_port_channel_interface_1.current.uuid }}'
      side_b: true
    encapsulation_type: vlan
    encapsulation_value: 1000
    parent_type: svi
    secondary_address: 10.0.2.4/24
    state: present

- name: Create a secondary address on a L3out Routed Sub-Interface of type port
  cisco.mso.ndo_l3out_secondary_ip:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: l3out_template
    l3out: l3out_1
    node_id: 101
    path: eth1/2
    encapsulation_type: vlan
    encapsulation_value: 101
    parent_type: routed_sub
    secondary_address: 10.1.0.2/24
    state: present

- name: Create a secondary address on a L3out Routed Sub-Interface of type port channel
  cisco.mso.ndo_l3out_secondary_ip:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: l3out_template
    l3out: l3out_1
    port_channel:
      uuid: '{{ port_channel_interface_2.current.uuid }}'
    encapsulation_type: vlan
    encapsulation_value: 101
    parent_type: routed_sub
    secondary_address: 2::102/16
    state: present

- name: Create a secondary address on a L3Out Floating SVI Interface
  cisco.mso.ndo_l3out_secondary_ip:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: l3out_template
    l3out: l3out_1
    node_id: 101
    encapsulation_type: vlan
    encapsulation_value: 100
    parent_type: floating_svi
    secondary_address: 10.4.0.3/24
    state: present

- name: Create a secondary address on a L3Out Floating SVI Interface path attributes
  cisco.mso.ndo_l3out_secondary_ip:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: l3out_template
    l3out: l3out_1
    node_id: 101
    encapsulation_type: vlan
    encapsulation_value: 100
    domain_type: vmm
    domain_provider: vmware
    domain: ansible_test_vmm
    parent_type: floating_svi_path_attributes
    secondary_address: 10.4.0.4/24
    state: present

- name: Update a secondary address on a L3out Routed Interface of type port
  cisco.mso.ndo_l3out_secondary_ip:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: l3out_template
    l3out: l3out_1
    node_id: 101
    path: eth1/3
    parent_type: routed
    secondary_address: 3::102/16
    dhcp_relay: true
    ipv6_dad: disabled
    nd_ra_prefix: true

- name: Get a secondary address on a L3out Routed Interface of type port
  cisco.mso.ndo_l3out_secondary_ip:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: l3out_template
    l3out: l3out_1
    node_id: 101
    path: eth1/3
    parent_type: routed
    secondary_address: 3::102/16
    state: query
  register: query_secondary_address

- name: Get all secondary addresses on a L3out Routed Interface of type port
  cisco.mso.ndo_l3out_secondary_ip:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: l3out_template
    l3out: l3out_1
    node_id: 101
    path: eth1/3
    parent_type: routed
    state: query
  register: query_all

- name: Delete a secondary address on a L3out Routed Interface of type port
  cisco.mso.ndo_l3out_secondary_ip:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: l3out_template
    l3out: l3out_1
    node_id: 101
    path: eth1/3
    parent_type: routed
    secondary_address: 3::102/16
    state: absent
"""

RETURN = r"""
"""


import copy
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import (
    MSOModule,
    mso_argument_spec,
    ndo_l3out_port_channel_spec,
    ndo_l3out_virtual_port_channel_spec,
)
from ansible_collections.cisco.mso.plugins.module_utils.templates import MSOTemplates
from ansible_collections.cisco.mso.plugins.module_utils.constants import DOMAIN_TYPE_MAP, VM_DOMAIN_PROVIDER_MAP
from ansible_collections.cisco.mso.plugins.module_utils.utils import append_update_ops_data


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        template=dict(type="str", aliases=["l3out_template"]),
        template_id=dict(type="str", aliases=["l3out_template_id"]),
        l3out=dict(type="str"),
        l3out_uuid=dict(type="str"),
        parent_type=dict(type="str", choices=["floating_svi", "routed", "routed_sub", "svi", "floating_svi_path_attributes"], required=True),
        node_id=dict(type="str", aliases=["node", "border_leaf", "anchor_node"]),
        path=dict(type="str", aliases=["interface"]),
        port_channel=ndo_l3out_port_channel_spec(micro_bfd=False),
        virtual_port_channel=ndo_l3out_virtual_port_channel_spec(side_b=False, secondary_address=True),
        encapsulation_type=dict(type="str", choices=["vlan", "vxlan"], aliases=["encap_type"]),
        encapsulation_value=dict(type="int", aliases=["encap", "encapsulation", "encapsulation_id"]),
        domain_type=dict(type="str", choices=list(DOMAIN_TYPE_MAP)),
        domain_provider=dict(type="str", choices=list(VM_DOMAIN_PROVIDER_MAP)),
        domain=dict(type="str", aliases=["domain_name"]),
        secondary_address=dict(type="str", aliases=["ip", "ipv4", "ipv6"]),
        dhcp_relay=dict(type="bool"),
        nd_ra_prefix=dict(type="bool"),
        ipv6_dad=dict(type="str", choices=["enabled", "disabled"]),
        state=dict(type="str", default="query", choices=["absent", "query", "present"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "present", ["secondary_address"]],
            ["state", "absent", ["secondary_address"]],
            ["parent_type", "routed", ["path", "port_channel"], True],
            ["parent_type", "routed_sub", ["path", "port_channel"], True],
            ["parent_type", "routed_sub", ["encapsulation_type", "encapsulation_value"]],
            ["parent_type", "svi", ["path", "port_channel", "virtual_port_channel"], True],
            ["parent_type", "svi", ["encapsulation_type", "encapsulation_value"]],
            ["parent_type", "floating_svi", ["node_id", "encapsulation_type", "encapsulation_value"]],
            ["parent_type", "floating_svi_path_attributes", ["node_id", "encapsulation_type", "encapsulation_value", "domain", "domain_type"]],
        ],
        required_by={"path": ("node_id")},
        mutually_exclusive=[
            ("template", "template_id"),
            ("l3out", "l3out_uuid"),
            ("port_channel", "path"),
            ("port_channel", "node_id"),
            ("port_channel", "virtual_port_channel"),
            ("virtual_port_channel", "path"),
            ("virtual_port_channel", "node_id"),
        ],
        required_one_of=[
            ["template", "template_id"],
            ["l3out", "l3out_uuid"],
        ],
    )

    mso = MSOModule(module)
    mso_templates = MSOTemplates(mso)

    template = module.params.get("template")
    template_id = module.params.get("template_id")
    l3out = module.params.get("l3out")
    l3out_uuid = module.params.get("l3out_uuid")
    parent_type = module.params.get("parent_type")
    secondary_address = module.params.get("secondary_address")
    dhcp_relay = module.params.get("dhcp_relay")
    nd_ra_prefix = module.params.get("nd_ra_prefix")
    ipv6_dad = module.params.get("ipv6_dad")
    side_b = None
    virtual_port_channel = module.params.get("virtual_port_channel")
    if virtual_port_channel:
        side_b = virtual_port_channel.get("side_b")

    state = module.params.get("state")

    mso_template = mso_templates.get_template("l3out", template, template_id)
    mso_template.validate_template("l3out")

    l3out_object = mso_template.get_l3out_object(uuid=l3out_uuid, name=l3out, fail_module=True)
    parent_object, parent_path = mso_template.get_parent_details_for_nested_object_in_l3out(mso_templates, l3out_object)
    match = mso_template.get_l3out_secondary_address(
        parent_object.details,
        parent_type,
        secondary_address,
        side_b,
    )
    if match and (secondary_address):
        set_secondary_ip_details(mso_template, parent_type, parent_object.details, match.details, side_b)
        mso.existing = mso.previous = copy.deepcopy(match.details)
    elif match:
        mso.existing = [
            set_secondary_ip_details(mso_template, parent_type, parent_object.details, bgp_peer, side_b) for bgp_peer in match
        ]  # Query all objects

    if parent_type == "floating_svi_path_attributes":
        secondary_path = "secondaryAddresses"
    elif side_b:
        secondary_path = "sideBAddresses/secondary"
    else:
        secondary_path = "addresses/secondary"

    secondary_ip_path = "{0}/{1}/{2}".format(parent_path, secondary_path, match.index if match else "-")
    ops = []

    if state == "present":
        mso_values = {"address": secondary_address, "dhcpRelay": dhcp_relay, "v6RAPrefix": nd_ra_prefix, "ipV6DAD": ipv6_dad}
        mso.sanitize(mso_values)
        if match:
            append_update_ops_data(ops, match.details, secondary_ip_path, mso_values)
            mso.sanitize(match.details, collate=True)
        else:
            mso.sanitize(mso_values)
            ops.append(dict(op="add", path=secondary_ip_path, value=mso.sent))
        set_secondary_ip_details(mso_template, parent_type, parent_object.details, mso.proposed, side_b)

    elif state == "absent":
        if mso.existing:
            ops.append(dict(op="remove", path=secondary_ip_path))

    if not module.check_mode and ops:
        mso_template.template = mso.request(mso_template.template_path, method="PATCH", data=ops)
        l3out_object = mso_template.get_l3out_object(uuid=l3out_uuid, name=l3out, fail_module=True)
        parent_object = mso_template.get_parent_details_for_nested_object_in_l3out(mso_templates, l3out_object)
        match = mso_template.get_l3out_secondary_address(parent_object[0].details, parent_type, secondary_address, side_b)
        if match:
            set_secondary_ip_details(mso_template, parent_type, parent_object[0].details, match.details, side_b)
            mso.existing = match.details  # When the state is present
        else:
            mso.existing = {}  # When the state is absent
    elif module.check_mode and state != "query":  # When the state is present/absent with check mode
        mso.existing = mso.proposed if state == "present" else {}
    mso.exit_json()


def set_secondary_ip_details(mso_template, parent_type, parent_object, match_object, side_b):
    if side_b is not None:
        match_object["virtualPortChannelSide"] = "B" if side_b else "A"
    mso_template.update_config_with_template_and_references(match_object)
    mso_template.set_parent_details_for_nested_object_in_l3out(parent_type, parent_object, match_object)
    return match_object


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2023, Dag Wieers (@dagwieers)
# Copyright: (c) 2023, Tim Cragg (@timcragg) <tcragg@cisco.com>
# Copyright: (c) 2023, Gaspard Micol (@gmicol) <gmicol@cisco.com>
# Copyright: (c) 2025, Shreyas Srish (@shrsr) <ssrish@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "certified"}

DOCUMENTATION = r"""
---
module: aci_tenant_action_rule_profile
short_description: Manage action rule profiles (rtctrl:AttrP)
description:
- Manage action rule profiles on Cisco ACI fabrics.
options:
  tenant:
    description:
    - The name of the tenant.
    type: str
    aliases: [ tenant_name ]
  action_rule:
    description:
    - The name of the action rule profile.
    type: str
    aliases: [action_rule_name, name ]
  set_community:
    description:
    - The set action rule based on communities.
    - To delete this attribute, pass an empty dictionary.
    type: dict
    suboptions:
      community:
        description:
        - The community value.
        type: str
      criteria:
        description:
        - The community criteria.
        - The option to append or replace the community value.
        type: str
        choices: [ append, replace, none ]
  set_dampening:
    description:
    - The set action rule based on dampening.
    - To delete this attribute, pass an empty dictionary.
    type: dict
    suboptions:
      half_life:
        description:
        - The half life value (minutes).
        type: int
      max_suppress_time:
        description:
        - The maximum suppress time value (minutes).
        type: int
      reuse:
        description:
        - The reuse limit value.
        type: int
      suppress:
        description:
        - The suppress limit value.
        type: int
  set_next_hop:
    description:
    - The set action rule based on the next hop address.
    - To delete this attribute, pass an empty string.
    type: str
  next_hop_propagation:
    description:
    - The set action rule based on nexthop unchanged configuration.
    - Can not be configured along with C(set_route_tag).
    - Can not be configured for APIC version 4.2 and prior.
    - The APIC defaults to C(false) when unset.
    type: bool
  multipath:
    description:
    - Set action rule based on set redistribute multipath configuration.
    - Can not be configured along with C(set_route_tag).
    - Can not be configured for APIC version 4.2 and prior.
    - The APIC defaults to C(false) when unset.
    type: bool
  set_preference:
    description:
    - The set action rule based on preference.
    - To delete this attribute, pass an empty string.
    type: str
  set_metric:
    description:
    - The set action rule based on metric.
    - To delete this attribute, pass an empty string.
    type: str
  set_metric_type:
    description:
    - The set action rule based on a metric type.
    - To delete this attribute, pass an empty string.
    type: str
    choices: [ ospf_type_1, ospf_type_2, "" ]
  set_route_tag:
    description:
    - The set action rule based on route tag.
    - Can not be configured along with C(next_hop_propagation) and C(multipath).
    - To delete this attribute, pass an empty string.
    type: str
  set_weight:
    description:
    - The set action rule based on weight.
    - To delete this attribute, pass an empty string.
    type: str
  set_communities:
    description:
    - List of additional communities to add to the action rule profile using the append criteria.
    - Providing an empty list O(set_communities=[]) will remove all communities.
    - This is only supported in Cisco ACI Release 6.0(2) and higher.
    type: list
    elements: dict
    suboptions:
      community:
        description:
        - The community value (e.g., no-advertise, no-export, or regular:as2-nn2:4:15).
        type: str
        required: true
      description:
        description:
        - Description for the community.
        type: str
        aliases: [ descr ]
  set_as_path:
    description:
    - List of Autonomous System (AS) path prepend configurations.
    - Providing an empty list O(set_as_path=[]) will remove all AS path configurations.
    - This is only supported in Cisco ACI Release 6.0(2) and higher.
    type: list
    elements: dict
    suboptions:
      criteria:
        description:
        - The AS path criteria.
        type: str
        choices: [ prepend, prepend-last-as ]
      last_num:
        description:
        - Number of times to prepend the last AS.
        - Only applicable when criteria is prepend-last-as.
        type: int
      asns:
        description:
        - List of ASNs to prepend.
        - Only applicable when criteria is prepend.
        type: list
        elements: dict
        suboptions:
          asn:
            description:
            - The ASN to prepend.
            type: str
            required: true
          order:
            description:
            - The order of the ASN in the prepend list.
            type: int
            required: true
  set_policy_tag:
    description:
    - The set action rule based on policy tag (External EPG or ESG).
    - Providing an empty dictionary O(set_policy_tag={}) will remove this attribute.
    - Either External EPG (l3out and external_epg) or ESG (ap and esg) must be configured, but not both.
    - This is only supported in Cisco ACI Release 6.0(2) and higher.
    type: dict
    suboptions:
      l3out:
        description:
        - The name of the L3Out.
        - Required together with C(external_epg).
        - Mutually exclusive with C(ap) and C(esg).
        type: str
      external_epg:
        description:
        - The name of the External EPG.
        - Required together with C(l3out).
        - Mutually exclusive with C(ap) and C(esg).
        type: str
        aliases: [ extepg, inst_p ]
      ap:
        description:
        - The name of the Application Profile.
        - Required together with C(esg).
        - Mutually exclusive with C(l3out) and C(external_epg).
        type: str
        aliases: [ app_profile ]
      esg:
        description:
        - The name of the Endpoint Security Group (ESG).
        - Required together with C(ap).
        - Mutually exclusive with C(l3out) and C(external_epg).
        type: str
  description:
    description:
    - The description for the action rule profile.
    type: str
    aliases: [ descr ]
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
  name_alias:
    description:
    - The alias for the current object. This relates to the nameAlias field in ACI.
    type: str
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation

notes:
- The C(tenant) used must exist before using this module in your playbook.
  The M(cisco.aci.aci_tenant) module can be used for this.
seealso:
- module: cisco.aci.aci_tenant
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(rtctrl:AttrP).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Dag Wieers (@dagwieers)
- Tim Cragg (@timcragg)
- Gaspard Micol (@gmicol)
- Shreyas Srish (@shrsr)
"""

EXAMPLES = r"""
- name: Create a action rule profile (with External EPG policy tag)
  cisco.aci.aci_tenant_action_rule_profile:
    host: apic
    username: admin
    password: SomeSecretPassword
    action_rule: my_action_rule
    tenant: prod
    set_preference: 100
    set_weight: 100
    set_metric: 100
    set_metric_type: ospf_type_1
    set_next_hop: 1.1.1.1
    next_hop_propagation: true
    multipath: true
    set_community:
      community: no-advertise
      criteria: replace
    set_dampening:
      half_life: 10
      reuse: 1
      suppress: 10
      max_suppress_time: 100
    set_communities:
      - community: no-advertise
        description: test
      - community: no-export
        description: test2
    set_as_path:
      - criteria: prepend
        asns:
          - asn: "65001"
            order: 0
          - asn: "65002"
            order: 1
      - criteria: prepend-last-as
        last_num: 4
    set_policy_tag:
      l3out: test
      external_epg: test
    state: present
  delegate_to: localhost

- name: Create a action rule profile (with ESG policy tag)
  cisco.aci.aci_tenant_action_rule_profile:
    host: apic
    username: admin
    password: SomeSecretPassword
    action_rule: my_action_rule
    tenant: Test
    set_communities:
      - community: no-advertise
        description: test
      - community: no-export
        description: test2
    set_as_path:
      - criteria: prepend
        asns:
          - asn: "65001"
            order: 0
          - asn: "65002"
            order: 1
      - criteria: prepend-last-as
        last_num: 4
    set_policy_tag:
      ap: ap
      esg: test
    state: present
  delegate_to: localhost

- name: Delete action rule profile's children
  cisco.aci.aci_tenant_action_rule_profile:
    host: apic
    username: admin
    password: SomeSecretPassword
    action_rule: my_action_rule
    tenant: prod
    set_preference: ""
    set_weight: ""
    set_metric: ""
    set_metric_type: ""
    set_next_hop: ""
    next_hop_propagation: false
    multipath: false
    set_community: {}
    set_dampening: {}
    set_communities: []
    set_as_path: []
    set_policy_tag: {}
    state: present
  delegate_to: localhost

- name: Delete an action rule profile
  cisco.aci.aci_tenant_action_rule_profile:
    host: apic
    username: admin
    password: SomeSecretPassword
    action_rule: my_action_rule
    tenant: prod
    state: absent
  delegate_to: localhost

- name: Query all action rule profiles
  cisco.aci.aci_tenant_action_rule_profile:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost
  register: query_result

- name: Query a specific action rule profile
  cisco.aci.aci_tenant_action_rule_profile:
    host: apic
    username: admin
    password: SomeSecretPassword
    action_rule: my_action_rule
    tenant: prod
    state: query
  delegate_to: localhost
  register: query_result
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import (
    ACIModule,
    aci_argument_spec,
    aci_annotation_spec,
    action_rule_set_comm_spec,
    action_rule_set_dampening_spec,
)
from ansible_collections.cisco.aci.plugins.module_utils.constants import MATCH_ACTION_RULE_SET_METRIC_TYPE_MAPPING


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(
        action_rule=dict(type="str", aliases=["action_rule_name", "name"]),  # Not required for querying all objects
        tenant=dict(type="str", aliases=["tenant_name"]),  # Not required for querying all objects
        set_community=dict(type="dict", options=action_rule_set_comm_spec()),
        set_dampening=dict(type="dict", options=action_rule_set_dampening_spec()),
        set_next_hop=dict(type="str"),
        next_hop_propagation=dict(type="bool"),
        multipath=dict(type="bool"),
        set_preference=dict(type="str"),
        set_metric=dict(type="str"),
        set_metric_type=dict(type="str", choices=["ospf_type_1", "ospf_type_2", ""]),
        set_route_tag=dict(type="str"),
        set_weight=dict(type="str"),
        set_communities=dict(
            type="list",
            elements="dict",
            options=dict(
                community=dict(type="str", required=True),
                description=dict(type="str", aliases=["descr"]),
            ),
        ),
        set_as_path=dict(
            type="list",
            elements="dict",
            options=dict(
                criteria=dict(type="str", choices=["prepend", "prepend-last-as"]),
                last_num=dict(type="int"),
                asns=dict(
                    type="list",
                    elements="dict",
                    options=dict(
                        asn=dict(type="str", required=True),
                        order=dict(type="int", required=True),
                    ),
                ),
            ),
        ),
        set_policy_tag=dict(
            type="dict",
            options=dict(
                l3out=dict(type="str"),
                external_epg=dict(type="str", aliases=["extepg", "inst_p"]),
                ap=dict(type="str", aliases=["app_profile"]),
                esg=dict(type="str"),
            ),
            mutually_exclusive=[
                ["l3out", "ap"],
                ["l3out", "esg"],
                ["external_epg", "ap"],
                ["external_epg", "esg"],
            ],
            required_together=[
                ["l3out", "external_epg"],
                ["ap", "esg"],
            ],
        ),
        description=dict(type="str", aliases=["descr"]),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
        name_alias=dict(type="str"),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["action_rule", "tenant"]],
            ["state", "present", ["action_rule", "tenant"]],
        ],
    )

    action_rule = module.params.get("action_rule")
    description = module.params.get("description")
    state = module.params.get("state")
    tenant = module.params.get("tenant")
    name_alias = module.params.get("name_alias")

    aci = ACIModule(module)

    # This dict contains the name of the child classes as well as the corresping attribute input (and attribute name if the input is a string)
    # this dict is deviating from normal child classes list structure in order to determine which child classes should be created, modified, deleted or ignored.
    child_classes = dict(
        rtctrlSetComm=dict(attribute_input=module.params.get("set_community")),
        rtctrlSetDamp=dict(attribute_input=module.params.get("set_dampening")),
        rtctrlSetNh=dict(attribute_input=module.params.get("set_next_hop"), attribute_name="addr"),
        rtctrlSetPref=dict(attribute_input=module.params.get("set_preference"), attribute_name="localPref"),
        rtctrlSetRtMetric=dict(attribute_input=module.params.get("set_metric"), attribute_name="metric"),
        rtctrlSetRtMetricType=dict(
            attribute_input=MATCH_ACTION_RULE_SET_METRIC_TYPE_MAPPING.get(module.params.get("set_metric_type")), attribute_name="metricType"
        ),
        rtctrlSetTag=dict(attribute_input=module.params.get("set_route_tag"), attribute_name="tag"),
        rtctrlSetWeight=dict(attribute_input=module.params.get("set_weight"), attribute_name="weight"),
    )

    # This condition deal with child classes which do not exist in APIC version 4.2 and prior.
    additional_child_classes = dict(
        rtctrlSetNhUnchanged=dict(attribute_input=module.params.get("next_hop_propagation")),
        rtctrlSetRedistMultipath=dict(attribute_input=module.params.get("multipath")),
    )
    for class_name, attribute in additional_child_classes.items():
        if attribute.get("attribute_input") is not None:
            child_classes[class_name] = attribute

    # Add new child classes which do not exist in APIC version 5.8/6.0 and prior for list-based configurations
    list_child_classes = []
    if module.params.get("set_communities") is not None:
        list_child_classes.append("rtctrlSetAddComm")
    if module.params.get("set_as_path") is not None:
        list_child_classes.append("rtctrlSetASPath")
    if module.params.get("set_policy_tag") is not None:
        list_child_classes.append("rtctrlSetPolicyTag")

    aci.construct_url(
        root_class=dict(
            aci_class="fvTenant",
            aci_rn="tn-{0}".format(tenant),
            module_object=tenant,
            target_filter={"name": tenant},
        ),
        subclass_1=dict(
            aci_class="rtctrlAttrP",
            aci_rn="attr-{0}".format(action_rule),
            module_object=action_rule,
            target_filter={"name": action_rule},
        ),
        child_classes=list(child_classes.keys()) + list_child_classes,
    )

    aci.get_existing()

    if state == "present":
        child_configs = []

        # Process existing single-value child classes
        for class_name, attribute in child_classes.items():
            attribute_input = attribute.get("attribute_input")
            # This condition enables to user to keep its previous configurations if they are not passing anything in the payload.
            if attribute_input is not None:
                # This condition checks if the attribute input is a dict and checks if all of its values are None (stored as a boolean in only_none).
                only_none = False
                if isinstance(attribute_input, dict):
                    only_none = all(value is None for value in attribute_input.values())
                # This condition checks if the child object needs to be deleted depending on the type of the corresponding attribute input (bool, str, dict).
                if (attribute_input == "" or attribute_input is False or only_none) and isinstance(aci.existing, list) and len(aci.existing) > 0:
                    for child in aci.existing[0].get("rtctrlAttrP", {}).get("children", {}):
                        if child.get(class_name):
                            child_configs.append(
                                {
                                    class_name: dict(
                                        attributes=dict(status="deleted"),
                                    ),
                                }
                            )
                # This condition checks if the child object needs to be modified or created depending on the type of the corresponding attribute input.
                elif attribute_input != "" or attribute_input is True or attribute_input != {}:
                    if class_name == "rtctrlSetComm" and isinstance(attribute_input, dict):
                        child_configs.append(
                            {
                                class_name: dict(
                                    attributes=dict(
                                        community=attribute_input.get("community"),
                                        setCriteria=attribute_input.get("criteria"),
                                    ),
                                )
                            }
                        )
                    elif class_name == "rtctrlSetDamp" and isinstance(attribute_input, dict):
                        child_configs.append(
                            {
                                class_name: dict(
                                    attributes=dict(
                                        halfLife=attribute_input.get("half_life"),
                                        maxSuppressTime=attribute_input.get("max_suppress_time"),
                                        reuse=attribute_input.get("reuse"),
                                        suppress=attribute_input.get("suppress"),
                                    ),
                                )
                            }
                        )
                    elif class_name in ["rtctrlSetNhUnchanged", "rtctrlSetRedistMultipath"]:
                        child_configs.append({class_name: dict(attributes=dict(descr=""))})
                    else:
                        child_configs.append({class_name: dict(attributes={attribute.get("attribute_name"): attribute_input})})

        # Process rtctrlSetAddComm (list of communities)
        set_communities = module.params.get("set_communities")
        if set_communities is not None:
            existing_communities = []

            # Get existing communities from APIC
            if isinstance(aci.existing, list) and len(aci.existing) > 0:
                for child in aci.existing[0].get("rtctrlAttrP", {}).get("children", []):
                    if child.get("rtctrlSetAddComm"):
                        existing_communities.append(child["rtctrlSetAddComm"])

            # Get list of communities from user input
            new_communities = [comm.get("community") for comm in set_communities]

            # Delete communities that are no longer in the new list
            for community_obj in existing_communities:
                existing_community = community_obj.get("attributes", {}).get("community")
                if existing_community and existing_community not in new_communities:
                    child_configs.append(
                        {
                            "rtctrlSetAddComm": dict(
                                attributes=dict(
                                    community=existing_community,
                                    status="deleted",
                                ),
                            )
                        }
                    )

            # Add new communities
            for comm in set_communities:
                child_configs.append(
                    {
                        "rtctrlSetAddComm": dict(
                            attributes=dict(
                                community=comm.get("community"),
                                descr=comm.get("description"),
                            ),
                        )
                    }
                )

        # Process rtctrlSetASPath (list of AS path configurations)
        # Process rtctrlSetASPath (list of AS path configurations)
        set_as_path = module.params.get("set_as_path")
        if set_as_path is not None:
            existing_as_paths = []

            # Get existing AS paths from APIC
            if isinstance(aci.existing, list) and len(aci.existing) > 0:
                for child in aci.existing[0].get("rtctrlAttrP", {}).get("children", []):
                    if child.get("rtctrlSetASPath"):
                        existing_as_paths.append(child["rtctrlSetASPath"])

            # Organize AS path configurations
            prepend_config = None
            prepend_last_as_config = None

            for as_path_config in set_as_path:
                criteria = as_path_config.get("criteria")

                if criteria == "prepend-last-as":
                    prepend_last_as_config = as_path_config
                elif criteria == "prepend":
                    prepend_config = as_path_config

            # Build list of criteria identifiers that should exist in new config
            new_criteria = []
            if prepend_config:
                new_criteria.append("prepend")
            if prepend_last_as_config:
                new_criteria.append("prepend-last-as")

            # Delete AS paths that are no longer in the new list
            for as_path_obj in existing_as_paths:
                existing_criteria = as_path_obj.get("attributes", {}).get("criteria", "")

                if existing_criteria not in new_criteria:
                    # Build deletion config based on criteria
                    if existing_criteria == "prepend-last-as":
                        child_configs.append(
                            {
                                "rtctrlSetASPath": dict(
                                    attributes=dict(
                                        criteria="prepend-last-as",
                                        status="deleted",
                                    ),
                                )
                            }
                        )
                    else:
                        child_configs.append(
                            {
                                "rtctrlSetASPath": dict(
                                    attributes=dict(
                                        criteria="prepend",
                                        status="deleted",
                                    ),
                                )
                            }
                        )

            # Add/update prepend configuration
            if prepend_config:
                asns_list = prepend_config.get("asns", [])

                # Get existing ASNs for prepend (if it exists)
                existing_prepend_asns = []
                for as_path_obj in existing_as_paths:
                    existing_criteria = as_path_obj.get("attributes", {}).get("criteria")

                    if existing_criteria == "prepend":
                        # Get existing ASN children
                        existing_asn_children = as_path_obj.get("children", [])

                        for child in existing_asn_children:
                            if child.get("rtctrlSetASPathASN"):
                                asn = child["rtctrlSetASPathASN"]["attributes"].get("asn")
                                order = child["rtctrlSetASPathASN"]["attributes"].get("order")
                                if asn and order is not None:
                                    existing_prepend_asns.append({"asn": str(asn), "order": str(order)})
                        break

                # Build list of new ASN identifiers
                new_asn_identifiers = []
                for idx, asn_config in enumerate(asns_list):
                    asn = str(asn_config.get("asn"))
                    order = asn_config.get("order")
                    order = str(order)
                    new_asn_identifiers.append({"asn": asn, "order": order})

                # Delete ASNs that are no longer in the new list
                for existing_asn in existing_prepend_asns:
                    asn_found = False
                    for new_asn in new_asn_identifiers:
                        if existing_asn["asn"] == new_asn["asn"] and existing_asn["order"] == new_asn["order"]:
                            asn_found = True
                            break

                    if not asn_found:
                        # Delete this ASN
                        child_configs.append(
                            {
                                "rtctrlSetASPath": dict(
                                    attributes=dict(criteria="prepend"),
                                    children=[
                                        {
                                            "rtctrlSetASPathASN": dict(
                                                attributes=dict(
                                                    asn=existing_asn["asn"],
                                                    order=existing_asn["order"],
                                                    status="deleted",
                                                ),
                                            )
                                        }
                                    ],
                                )
                            }
                        )

                # Add new ASNs
                asn_children = []
                for idx, asn_config in enumerate(asns_list):
                    asn = asn_config.get("asn")
                    order = asn_config.get("order")

                    asn_children.append(
                        {
                            "rtctrlSetASPathASN": dict(
                                attributes=dict(
                                    asn=str(asn),
                                    order=str(order),
                                ),
                                children=[],
                            )
                        }
                    )

                as_path_attrs = dict(criteria="prepend")
                child_configs.append({"rtctrlSetASPath": dict(attributes=as_path_attrs, children=asn_children)})

            # Add/update prepend-last-as configuration
            if prepend_last_as_config:
                last_num = prepend_last_as_config.get("last_num")

                as_path_attrs = dict(
                    criteria="prepend-last-as",
                    lastnum=str(last_num),
                )
                child_configs.append({"rtctrlSetASPath": dict(attributes=as_path_attrs, children=[])})

        # Process rtctrlSetPolicyTag (External EPG or ESG reference)
        # Only one type can be configured at a time (mutually exclusive)
        set_policy_tag = module.params.get("set_policy_tag")
        if set_policy_tag is not None:
            existing_policy_tags = []

            # Get existing policy tags from APIC
            if isinstance(aci.existing, list) and len(aci.existing) > 0:
                for child in aci.existing[0].get("rtctrlAttrP", {}).get("children", []):
                    if child.get("rtctrlSetPolicyTag"):
                        existing_policy_tags.append(child["rtctrlSetPolicyTag"])

            # Check if all values are None (empty dict behavior)
            all_none = all(value is None for value in set_policy_tag.values()) if set_policy_tag else True
            # If empty dict is passed (all values are None), delete all existing policy tags
            if all_none:
                if existing_policy_tags:
                    child_configs.append(
                        {
                            "rtctrlSetPolicyTag": dict(
                                attributes=dict(status="deleted"),
                            )
                        }
                    )
            else:
                # Determine which type is being configured
                l3out = set_policy_tag.get("l3out")
                external_epg = set_policy_tag.get("external_epg")
                ap = set_policy_tag.get("ap")
                esg = set_policy_tag.get("esg")

                new_target_dn = None
                new_relationship_class = None

                # External EPG configuration
                if l3out and external_epg:
                    new_target_dn = "uni/tn-{0}/out-{1}/instP-{2}".format(tenant, l3out, external_epg)
                    new_relationship_class = "rtctrlRsSetPolicyTagToInstP"
                # ESG configuration
                elif ap and esg:
                    new_target_dn = "uni/tn-{0}/ap-{1}/esg-{2}".format(tenant, ap, esg)
                    new_relationship_class = "rtctrlRsSetPolicyTagToESg"

                # If we have valid new configuration
                if new_target_dn and new_relationship_class:
                    needs_update = False

                    # Check if existing policy tag matches the new configuration
                    if existing_policy_tags:
                        existing_policy_tag = existing_policy_tags[0]
                        existing_children = existing_policy_tag.get("children", [])

                        # Check if the same relationship with same target exists
                        existing_match = False
                        for rel_child in existing_children:
                            if rel_child.get(new_relationship_class):
                                existing_target_dn = rel_child[new_relationship_class]["attributes"].get("tDn")
                                if existing_target_dn == new_target_dn:
                                    existing_match = True
                                break

                        # If existing doesn't match, we need to update
                        if not existing_match:
                            needs_update = True
                            aci.api_call(
                                "DELETE",
                                "{0}/api/mo/uni/tn-{1}/attr-{2}/sptag.json".format(aci.base_url, tenant, action_rule),
                            )
                    else:
                        # No existing policy tag, need to create
                        needs_update = True

                    # Add new policy tag configuration if update is needed
                    if needs_update:
                        policy_tag_children = [
                            {
                                new_relationship_class: dict(
                                    attributes=dict(tDn=new_target_dn),
                                    children=[],
                                )
                            }
                        ]
                        child_configs.append(
                            {
                                "rtctrlSetPolicyTag": dict(
                                    attributes=dict(),
                                    children=policy_tag_children,
                                )
                            }
                        )

        aci.payload(
            aci_class="rtctrlAttrP",
            class_config=dict(
                name=action_rule,
                descr=description,
                nameAlias=name_alias,
            ),
            child_configs=child_configs,
        )

        aci.get_diff(aci_class="rtctrlAttrP")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_port_policy_info
short_description: Gather information about Cisco Intersight Port Policies
description:
  - Gathers information about Port Policies and their associated resources on Cisco Intersight.
  - Retrieves comprehensive information including breakout ports, server ports, uplink port channels, pin groups, and all other port configurations.
  - Supports filtering by policy name and organization.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs/fabric/PortPolicies/get/).
extends_documentation_fragment: intersight
options:
  name:
    description:
      - Name of the Port Policy to retrieve information for.
      - If not provided, information for all Port Policies will be returned.
    type: str
  organization:
    description:
      - The name of the Organization to filter policies by.
      - If not provided, policies from all accessible organizations will be returned.
    type: str
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
- name: Get information about all Port Policies
  cisco.intersight.intersight_port_policy_info:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
  register: all_port_policies

- name: Get information about a specific Port Policy
  cisco.intersight.intersight_port_policy_info:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "port-policy-example"
  register: specific_policy

- name: Get information about Port Policies in a specific organization
  cisco.intersight.intersight_port_policy_info:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "production"
  register: org_policies

- name: Get information about a specific Port Policy in a specific organization
  cisco.intersight.intersight_port_policy_info:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "port-policy-example"
    organization: "production"
  register: specific_org_policy
'''

RETURN = r'''
api_response:
  description: List of Port Policy information with all associated resources
  returned: always
  type: list
  elements: dict
  sample: [
    {
      "Name": "port-policy-example",
      "Moid": "5dee1d736972652d321d26b5",
      "ObjectType": "fabric.PortPolicy",
      "DeviceModel": "UCS-FI-6454",
      "Description": "Example port policy with various configurations",
      "Organization": {
        "Name": "default",
        "Moid": "5dee1d736972652d321d26b5",
        "ObjectType": "organization.Organization"
      },
      "Tags": [
        {
          "Key": "Environment",
          "Value": "Production"
        }
      ],
      "BreakoutPorts": [
        {
          "PortId": 49,
          "CustomMode": "BreakoutEthernet25G",
          "SlotId": 1,
          "Moid": "5dee1d736972652d321d26b6",
          "ObjectType": "fabric.PortMode"
        }
      ],
      "FcPortModes": [
        {
          "PortIdStart": 1,
          "PortIdEnd": 16,
          "SlotId": 1,
          "CustomMode": "FibreChannel",
          "Moid": "5dee1d736972652d321d26b7",
          "ObjectType": "fabric.PortMode"
        }
      ],
      "ServerRoles": [
        {
          "PortId": 1,
          "AggregatePortId": 49,
          "SlotId": 1,
          "Fec": "Auto",
          "UserLabel": "Server Port 1",
          "PreferredDeviceType": "Chassis",
          "PreferredDeviceId": 1,
          "Moid": "5dee1d736972652d321d26b8",
          "ObjectType": "fabric.ServerRole"
        }
      ],
      "UplinkPcRoles": [
        {
          "PcId": 123,
          "AdminSpeed": "25Gbps",
          "Fec": "Auto",
          "UserLabel": "Uplink PC 123",
          "SlotId": 1,
          "Moid": "5dee1d736972652d321d26b9",
          "ObjectType": "fabric.UplinkPcRole",
          "Ports": [
            {
              "PortId": 1,
              "SlotId": 1,
              "AggregatePortId": 0
            },
            {
              "PortId": 2,
              "SlotId": 1,
              "AggregatePortId": 0
            }
          ],
          "EthNetworkGroupPolicy": [
            {
              "Moid": "5dee1d736972652d321d26ba",
              "ObjectType": "fabric.EthNetworkGroupPolicy"
            }
          ],
          "FlowControlPolicy": {
            "Moid": "5dee1d736972652d321d26bb",
            "ObjectType": "fabric.FlowControlPolicy"
          },
          "LinkAggregationPolicy": {
            "Moid": "5dee1d736972652d321d26bc",
            "ObjectType": "fabric.LinkAggregationPolicy"
          },
          "LinkControlPolicy": {
            "Moid": "5dee1d736972652d321d26bd",
            "ObjectType": "fabric.LinkControlPolicy"
          }
        }
      ],
      "FcUplinkPortChannels": [
        {
          "PcId": 13,
          "AdminSpeed": "16Gbps",
          "VsanId": 1,
          "UserLabel": "FC Uplink PC 13",
          "SlotId": 1,
          "Moid": "5dee1d736972652d321d26be",
          "ObjectType": "fabric.FcUplinkPcRole",
          "Ports": [
            {
              "PortId": 17,
              "SlotId": 1,
              "AggregatePortId": 0
            },
            {
              "PortId": 18,
              "SlotId": 1,
              "AggregatePortId": 0
            }
          ]
        }
      ],
      "FcUplinkRoles": [
        {
          "PortId": 19,
          "AdminSpeed": "16Gbps",
          "VsanId": 1,
          "UserLabel": "FC Uplink 19",
          "SlotId": 1,
          "Moid": "5dee1d736972652d321d26bf",
          "ObjectType": "fabric.FcUplinkRole"
        }
      ],
      "UplinkRoles": [
        {
          "PortId": 3,
          "AdminSpeed": "25Gbps",
          "Fec": "Auto",
          "UserLabel": "Ethernet Uplink 3",
          "SlotId": 1,
          "Moid": "5dee1d736972652d321d26c0",
          "ObjectType": "fabric.UplinkRole"
        }
      ],
      "LanPinGroups": [
        {
          "Name": "pin-group-1",
          "Moid": "5dee1d736972652d321d26c1",
          "ObjectType": "fabric.LanPinGroup",
          "PinTargetInterfaceRole": {
            "Moid": "5dee1d736972652d321d26c2",
            "ObjectType": "fabric.UplinkPcRole"
          }
        }
      ],
      "SanPinGroups": [
        {
          "Name": "san-pin-group-1",
          "Moid": "5dee1d736972652d321d26c3",
          "ObjectType": "fabric.SanPinGroup",
          "PinTargetInterfaceRole": {
            "Moid": "5dee1d736972652d321d26c4",
            "ObjectType": "fabric.FcUplinkPcRole"
          }
        }
      ],
      "FcoeUplinkPcRoles": [],
      "AppliancePcRoles": [],
      "FcStorageRoles": [],
      "ApplianceRoles": [],
      "FcoeUplinkRoles": []
    }
  ]
'''

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


def get_port_policy_secondary_resources(intersight, port_policy_moid):
    """
    Retrieve all secondary resources associated with a port policy.

    Args:
        intersight: IntersightModule instance
        port_policy_moid: MOID of the port policy

    Returns:
        Dictionary containing all secondary resources
    """
    # Initialize all resource keys to ensure consistent structure
    secondary_resources = {
        'BreakoutPorts': [],
        'FcPortModes': [],
        'ServerRoles': [],
        'UplinkPcRoles': [],
        'FcUplinkPortChannels': [],
        'FcoeUplinkPcRoles': [],
        'AppliancePcRoles': [],
        'FcUplinkRoles': [],
        'FcStorageRoles': [],
        'ApplianceRoles': [],
        'UplinkRoles': [],
        'FcoeUplinkRoles': [],
        'LanPinGroups': [],
        'SanPinGroups': [],
    }

    # Define resource mappings based on actual endpoints used in intersight_port_policy.py
    resource_mappings = [
        # Port Modes (FC port mode and Breakout ports)
        ('/fabric/PortModes', 'PortModes'),

        # Port Channels
        ('/fabric/UplinkPcRoles', 'UplinkPcRoles'),
        ('/fabric/FcUplinkPcRoles', 'FcUplinkPortChannels'),
        ('/fabric/FcoeUplinkPcRoles', 'FcoeUplinkPcRoles'),
        ('/fabric/AppliancePcRoles', 'AppliancePcRoles'),

        # Individual Ports
        ('/fabric/ServerRoles', 'ServerRoles'),
        ('/fabric/FcUplinkRoles', 'FcUplinkRoles'),
        ('/fabric/FcStorageRoles', 'FcStorageRoles'),
        ('/fabric/ApplianceRoles', 'ApplianceRoles'),
        ('/fabric/UplinkRoles', 'UplinkRoles'),
        ('/fabric/FcoeUplinkRoles', 'FcoeUplinkRoles'),

        # Pin Groups
        ('/fabric/LanPinGroups', 'LanPinGroups'),
        ('/fabric/SanPinGroups', 'SanPinGroups'),
    ]

    for api_path, result_key in resource_mappings:
        try:
            # Build query parameters
            query_params = {
                '$filter': f"PortPolicy.Moid eq '{port_policy_moid}'"
            }
            intersight.result['api_response'] = {}
            # Get the resources
            intersight.get_resource(
                resource_path=api_path,
                query_params=query_params,
                return_list=True
            )

            if intersight.result.get('api_response'):
                resources = intersight.result['api_response']

                # Special handling for PortModes to separate FC port mode and breakout ports
                if result_key == 'PortModes':
                    fc_port_modes = []
                    breakout_ports = []

                    for port_mode in resources:
                        # Make a deep copy to avoid modifying original data
                        port_mode_copy = port_mode.copy()

                        # Breakout ports have CustomMode specified
                        if port_mode_copy.get('CustomMode') != "FibreChannel":
                            # Change PortIdStart and PortIdEnd to PortId to align the main module.
                            if 'PortIdStart' in port_mode_copy:
                                port_mode_copy['PortId'] = port_mode_copy['PortIdStart']
                                del port_mode_copy['PortIdStart']
                            if 'PortIdEnd' in port_mode_copy:
                                del port_mode_copy['PortIdEnd']
                            breakout_ports.append(port_mode_copy)
                        else:
                            fc_port_modes.append(port_mode_copy)

                    secondary_resources['FcPortModes'] = fc_port_modes
                    secondary_resources['BreakoutPorts'] = breakout_ports
                else:
                    secondary_resources[result_key] = resources

        except Exception as e:
            # Log the error but continue with other resources
            intersight.module.warn(f"Failed to retrieve {result_key} from {api_path}: {str(e)}")

    return secondary_resources


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        name=dict(type='str'),
        organization=dict(type='str')
    )

    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )

    intersight = IntersightModule(module)

    # Build query parameters using the standard method
    query_params = intersight.set_query_params()

    try:
        # Get Port Policies
        intersight.get_resource(
            resource_path='/fabric/PortPolicies',
            query_params=query_params,
            return_list=True
        )

        port_policies = intersight.result.get('api_response', [])

        # Ensure port_policies is a list
        if isinstance(port_policies, dict):
            port_policies = [port_policies]
        elif not isinstance(port_policies, list):
            port_policies = []

        # For each port policy, get all associated secondary resources
        for policy in port_policies:
            policy_moid = policy.get('Moid')
            if policy_moid:
                # Get all secondary resources for this policy
                secondary_resources = get_port_policy_secondary_resources(intersight, policy_moid)

                # Merge secondary resources directly into the policy data
                policy.update(secondary_resources)

        # Update the api_response with the enhanced policy data
        intersight.result['api_response'] = port_policies

        # Set count of policies found
        intersight.result['count'] = len(port_policies)

    except Exception as e:
        module.fail_json(msg=f"Failed to retrieve Port Policy information: {str(e)}")

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



# Copyright (c) 2018 Remi Verchere <remi@verchere.fr>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Make coding more python3-ish
from __future__ import annotations

DOCUMENTATION = r"""
name: nrdp
type: notification
author: "Remi VERCHERE (@rverchere)"
short_description: Post task results to a Nagios server through nrdp
description:
  - This callback send playbook result to Nagios.
  - Nagios shall use NRDP to receive passive events.
  - The passive check is sent to a dedicated host/service for Ansible.
options:
  url:
    description: URL of the nrdp server.
    required: true
    env:
      - name: NRDP_URL
    ini:
      - section: callback_nrdp
        key: url
    type: string
  validate_certs:
    description: Validate the SSL certificate of the nrdp server. (Used for HTTPS URLs).
    env:
      - name: NRDP_VALIDATE_CERTS
    ini:
      - section: callback_nrdp
        key: validate_nrdp_certs
      - section: callback_nrdp
        key: validate_certs
    type: boolean
    default: false
    aliases: [validate_nrdp_certs]
  token:
    description: Token to be allowed to push nrdp events.
    required: true
    env:
      - name: NRDP_TOKEN
    ini:
      - section: callback_nrdp
        key: token
    type: string
  hostname:
    description: Hostname where the passive check is linked to.
    required: true
    env:
      - name: NRDP_HOSTNAME
    ini:
      - section: callback_nrdp
        key: hostname
    type: string
  servicename:
    description: Service where the passive check is linked to.
    required: true
    env:
      - name: NRDP_SERVICENAME
    ini:
      - section: callback_nrdp
        key: servicename
    type: string
"""

from urllib.parse import urlencode

from ansible.module_utils.common.text.converters import to_bytes
from ansible.module_utils.urls import open_url
from ansible.plugins.callback import CallbackBase


class CallbackModule(CallbackBase):
    """
    send ansible-playbook to Nagios server using nrdp protocol
    """

    CALLBACK_VERSION = 2.0
    CALLBACK_TYPE = "notification"
    CALLBACK_NAME = "community.general.nrdp"
    CALLBACK_NEEDS_WHITELIST = True

    # Nagios states
    OK = 0
    WARNING = 1
    CRITICAL = 2
    UNKNOWN = 3

    def __init__(self):
        super().__init__()

        self.printed_playbook = False
        self.playbook_name = None
        self.play = None

    def set_options(self, task_keys=None, var_options=None, direct=None):
        super().set_options(task_keys=task_keys, var_options=var_options, direct=direct)

        self.url = self.get_option("url")
        if not self.url.endswith("/"):
            self.url += "/"
        self.token = self.get_option("token")
        self.hostname = self.get_option("hostname")
        self.servicename = self.get_option("servicename")
        self.validate_nrdp_certs = self.get_option("validate_certs")

        if (self.url or self.token or self.hostname or self.servicename) is None:
            self._display.warning(
                "NRDP callback wants the NRDP_URL,"
                " NRDP_TOKEN, NRDP_HOSTNAME,"
                " NRDP_SERVICENAME"
                " environment variables'."
                " The NRDP callback plugin is disabled."
            )
            self.disabled = True

    def _send_nrdp(self, state, msg):
        """
        nrpd service check send XMLDATA like this:
        <?xml version='1.0'?>
            <checkresults>
                <checkresult type='service'>
                    <hostname>somehost</hostname>
                    <servicename>someservice</servicename>
                    <state>1</state>
                    <output>WARNING: Danger Will Robinson!|perfdata</output>
                </checkresult>
            </checkresults>
        """
        xmldata = "<?xml version='1.0'?>\n"
        xmldata += "<checkresults>\n"
        xmldata += "<checkresult type='service'>\n"
        xmldata += f"<hostname>{self.hostname}</hostname>\n"
        xmldata += f"<servicename>{self.servicename}</servicename>\n"
        xmldata += f"<state>{state}</state>\n"
        xmldata += f"<output>{msg}</output>\n"
        xmldata += "</checkresult>\n"
        xmldata += "</checkresults>\n"

        body = {"cmd": "submitcheck", "token": self.token, "XMLDATA": to_bytes(xmldata)}

        try:
            response = open_url(self.url, data=urlencode(body), method="POST", validate_certs=self.validate_nrdp_certs)
            return response.read()
        except Exception as ex:
            self._display.warning(f"NRDP callback cannot send result {ex}")

    def v2_playbook_on_play_start(self, play):
        """
        Display Playbook and play start messages
        """
        self.play = play

    def v2_playbook_on_stats(self, stats):
        """
        Display info about playbook statistics
        """
        name = self.play
        gstats = ""
        hosts = sorted(stats.processed.keys())
        critical = warning = 0
        for host in hosts:
            stat = stats.summarize(host)
            gstats += (
                f"'{host}_ok'={stat['ok']} '{host}_changed'={stat['changed']}"
                f" '{host}_unreachable'={stat['unreachable']} '{host}_failed'={stat['failures']} "
            )
            # Critical when failed tasks or unreachable host
            critical += stat["failures"]
            critical += stat["unreachable"]
            # Warning when changed tasks
            warning += stat["changed"]

        msg = f"{name} | {gstats}"
        if critical:
            # Send Critical
            self._send_nrdp(self.CRITICAL, msg)
        elif warning:
            # Send Warning
            self._send_nrdp(self.WARNING, msg)
        else:
            # Send OK
            self._send_nrdp(self.OK, msg)



#!/usr/bin/python
# Copyright (c) 2024, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: x509_certificate_convert
short_description: Convert X.509 certificates
version_added: 2.19.0
description:
  - This module allows to convert X.509 certificates between different formats.
author:
  - Felix Fontein (@felixfontein)
extends_documentation_fragment:
  - ansible.builtin.files
  - community.crypto._attributes
  - community.crypto._attributes.files
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
  safe_file_operations:
    support: full
  idempotent:
    support: full
options:
  src_path:
    description:
      - Name of the file containing the X.509 certificate to convert.
      - Exactly one of O(src_path) or O(src_content) must be specified.
    type: path
  src_content:
    description:
      - The content of the file containing the X.509 certificate to convert.
      - This must be text. If you are not sure that the input file is PEM, you must Base64 encode the value and set O(src_content_base64=true).
        You can use the P(ansible.builtin.b64encode#filter) filter plugin for this.
      - Exactly one of O(src_path) or O(src_content) must be specified.
    type: str
  src_content_base64:
    description:
      - If set to V(true) when O(src_content) is provided, the module assumes that the value of O(src_content) is Base64 encoded.
    type: bool
    default: false
  format:
    description:
      - Determines which format the destination X.509 certificate should be written in.
      - Please note that not every key can be exported in any format, and that not every format supports encryption.
    type: str
    choices:
      - pem
      - der
    required: true
  strict:
    description:
      - If the input is a PEM file, ensure that it contains a single PEM object, that the header and footer match, and are
        of type C(CERTIFICATE) or C(X509 CERTIFICATE).
      - See also the O(verify_cert_parsable) option, which checks whether the certificate is parsable.
    type: bool
    default: false
  dest_path:
    description:
      - Name of the file in which the generated TLS/SSL X.509 certificate will be written.
    type: path
    required: true
  backup:
    description:
      - Create a backup file including a timestamp so you can get the original X.509 certificate back if you overwrote it
        with a new one by accident.
    type: bool
    default: false
  verify_cert_parsable:
    description:
      - If set to V(true), ensures that the certificate can be parsed.
      - To ensure that a PEM file does not contain multiple certificates, use the O(strict) option.
    type: bool
    default: false
    version_added: 2.23.0
seealso:
  - plugin: ansible.builtin.b64encode
    plugin_type: filter
  - module: community.crypto.x509_certificate
  - module: community.crypto.x509_certificate_pipe
  - module: community.crypto.x509_certificate_info
"""

EXAMPLES = r"""
---
- name: Convert PEM X.509 certificate to DER format
  community.crypto.x509_certificate_convert:
    src_path: /etc/ssl/cert/ansible.com.pem
    dest_path: /etc/ssl/cert/ansible.com.der
    format: der
"""

RETURN = r"""
backup_file:
  description: Name of backup file created.
  returned: changed and if O(backup) is V(true)
  type: str
  sample: /path/to/cert.pem.2019-03-09@11:22~
"""

import base64
import os
import typing as t

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.common.text.converters import to_bytes, to_text

from ansible_collections.community.crypto.plugins.module_utils._crypto.basic import (
    OpenSSLObjectError,
)
from ansible_collections.community.crypto.plugins.module_utils._crypto.pem import (
    PEM_END,
    PEM_END_START,
    PEM_START,
    extract_pem,
    identify_pem_format,
    split_pem_list,
)
from ansible_collections.community.crypto.plugins.module_utils._crypto.support import (
    OpenSSLObject,
)
from ansible_collections.community.crypto.plugins.module_utils._cryptography_dep import (
    COLLECTION_MINIMUM_CRYPTOGRAPHY_VERSION,
    assert_required_cryptography_version,
)
from ansible_collections.community.crypto.plugins.module_utils._io import (
    load_file_if_exists,
    write_file,
)

MINIMAL_CRYPTOGRAPHY_VERSION = COLLECTION_MINIMUM_CRYPTOGRAPHY_VERSION

try:
    from cryptography.x509 import load_der_x509_certificate
except ImportError:
    pass


def parse_certificate(
    certificate_content: bytes, strict: bool = False
) -> tuple[bytes, t.Literal["pem", "der"], str | None]:
    input_format: t.Literal["pem", "der"] = (
        "pem" if identify_pem_format(certificate_content) else "der"
    )
    if input_format == "pem":
        pems = split_pem_list(to_text(certificate_content))
        if len(pems) > 1 and strict:
            raise ValueError(
                f"The input contains {len(pems)} PEM objects, expecting only one since strict=true"
            )
        pem_header_type, content = extract_pem(pems[0], strict=strict)
        if strict and pem_header_type not in ("CERTIFICATE", "X509 CERTIFICATE"):
            raise ValueError(
                f"type is {pem_header_type!r}, expecting CERTIFICATE or X509 CERTIFICATE"
            )
        certificate_content = base64.b64decode(content)
    else:
        pem_header_type = None
    return certificate_content, input_format, pem_header_type


class X509CertificateConvertModule(OpenSSLObject):
    def __init__(self, module: AnsibleModule) -> None:
        super().__init__(
            path=module.params["dest_path"],
            state="present",
            force=False,
            check_mode=module.check_mode,
        )

        self.src_path: str | None = module.params["src_path"]
        self.src_content: str | None = module.params["src_content"]
        self.src_content_base64: bool = module.params["src_content_base64"]
        if self.src_content is not None:
            self.input = to_bytes(self.src_content)
            if self.src_content_base64:
                try:
                    self.input = base64.b64decode(self.input)
                except Exception as exc:
                    module.fail_json(msg=f"Cannot Base64 decode src_content: {exc}")
        else:
            if self.src_path is None:
                module.fail_json(msg="One of src_path and src_content must be provided")
            try:
                with open(self.src_path, "rb") as f:
                    self.input = f.read()
            except Exception as exc:
                module.fail_json(
                    msg=f"Failure while reading file {self.src_path}: {exc}"
                )

        self.format: t.Literal["pem", "der"] = module.params["format"]
        self.strict: bool = module.params["strict"]
        self.wanted_pem_type = "CERTIFICATE"

        try:
            self.input, self.input_format, dummy = parse_certificate(
                self.input, strict=self.strict
            )
        except Exception as exc:
            module.fail_json(msg=f"Error while parsing PEM: {exc}")

        if module.params["verify_cert_parsable"]:
            self.verify_cert_parsable(module)

        self.backup: bool = module.params["backup"]
        self.backup_file: str | None = None

        module.params["path"] = self.path

        self.dest_content = load_file_if_exists(path=self.path, module=module)
        self.dest_content_format = None
        self.dest_content_pem_type = None
        if self.dest_content is not None:
            try:
                (
                    self.dest_content,
                    self.dest_content_format,
                    self.dest_content_pem_type,
                ) = parse_certificate(self.dest_content, strict=True)
            except Exception:
                pass

    def verify_cert_parsable(self, module: AnsibleModule) -> None:
        assert_required_cryptography_version(
            module, minimum_cryptography_version=MINIMAL_CRYPTOGRAPHY_VERSION
        )
        try:
            load_der_x509_certificate(self.input)
        except Exception as exc:
            module.fail_json(msg=f"Error while parsing certificate: {exc}")

    def needs_conversion(self) -> bool:
        if self.dest_content is None or self.dest_content_format is None:
            return True
        if self.dest_content_format != self.format:
            return True
        if self.input != self.dest_content:
            return True
        return bool(
            self.format == "pem" and self.dest_content_pem_type != self.wanted_pem_type
        )

    def get_dest_certificate(self) -> bytes:
        if self.format == "der":
            return self.input
        data = to_bytes(base64.b64encode(self.input))
        lines = [to_bytes(f"{PEM_START}{self.wanted_pem_type}{PEM_END}")]
        lines += [data[i : i + 64] for i in range(0, len(data), 64)]
        lines.append(to_bytes(f"{PEM_END_START}{self.wanted_pem_type}{PEM_END}\n"))
        return b"\n".join(lines)

    def generate(self, module: AnsibleModule) -> None:
        """Do conversion."""
        if self.needs_conversion():
            # Convert
            cert_data = self.get_dest_certificate()
            if not self.check_mode:
                if self.backup:
                    self.backup_file = module.backup_local(self.path)
                write_file(module=module, content=cert_data)
            self.changed = True

        file_args = module.load_file_common_arguments(module.params)
        if module.check_file_absent_if_check_mode(file_args["path"]):
            self.changed = True
        else:
            self.changed = module.set_fs_attributes_if_different(
                file_args, self.changed
            )

    def dump(self) -> dict[str, t.Any]:
        """Serialize the object into a dictionary."""
        result: dict[str, t.Any] = {
            "changed": self.changed,
        }
        if self.backup_file:
            result["backup_file"] = self.backup_file

        return result


def main() -> t.NoReturn:
    argument_spec = {
        "src_path": {"type": "path"},
        "src_content": {"type": "str"},
        "src_content_base64": {"type": "bool", "default": False},
        "format": {"type": "str", "required": True, "choices": ["pem", "der"]},
        "strict": {"type": "bool", "default": False},
        "dest_path": {"type": "path", "required": True},
        "backup": {"type": "bool", "default": False},
        "verify_cert_parsable": {"type": "bool", "default": False},
    }
    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
        add_file_common_args=True,
        required_one_of=[("src_path", "src_content")],
        mutually_exclusive=[("src_path", "src_content")],
    )

    base_dir = os.path.dirname(module.params["dest_path"]) or "."
    if not os.path.isdir(base_dir):
        module.fail_json(
            name=base_dir,
            msg=f"The directory {base_dir} does not exist or the file is not a directory",
        )

    try:
        cert = X509CertificateConvertModule(module)
        cert.generate(module)
        result = cert.dump()
        module.exit_json(**result)
    except OpenSSLObjectError as exc:
        module.fail_json(msg=str(exc))


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "certified"}

DOCUMENTATION = r"""
---
module: aci_snmp_community_policy
short_description: Manage SNMP community policies (snmp:CommunityP)
description:
- Manage SNMP community policies
options:
  community:
    description:
    - Name of the SNMP community policy
    type: str
  description:
    description:
    - Description of the SNMP policy
    type: str
  policy:
    description:
    - Name of an existing SNMP policy
    type: str
    aliases: [ snmp_policy, snmp_policy_name ]
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation

seealso:
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(snmp:CommunityP).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Tim Cragg (@timcragg)
"""

EXAMPLES = r"""
- name: Create an SNMP community policy
  cisco.aci.aci_snmp_community_policy:
    host: apic
    username: admin
    password: SomeSecretPassword
    policy: my_snmp_policy
    community: my_snmp_community
    state: present
  delegate_to: localhost

- name: Remove an SNMP community policy
  cisco.aci.aci_snmp_community_policy:
    host: apic
    username: admin
    password: SomeSecretPassword
    policy: my_snmp_policy
    community: my_snmp_community
    state: absent
  delegate_to: localhost

- name: Query an SNMP community policy
  cisco.aci.aci_snmp_community_policy:
    host: apic
    username: admin
    password: SomeSecretPassword
    policy: my_snmp_policy
    community: my_snmp_community
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all SNMP community policies
  cisco.aci.aci_snmp_community_policy:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost
  register: query_result
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""


from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec
from ansible.module_utils.basic import AnsibleModule


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(
        community=dict(type="str"),
        policy=dict(type="str", aliases=["snmp_policy", "snmp_policy_name"]),
        description=dict(type="str"),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["policy", "community"]],
            ["state", "present", ["policy", "community"]],
        ],
    )

    aci = ACIModule(module)

    community = module.params.get("community")
    policy = module.params.get("policy")
    description = module.params.get("description")
    state = module.params.get("state")

    aci.construct_url(
        root_class=dict(
            aci_class="snmpPol",
            aci_rn="fabric/snmppol-{0}".format(policy),
            module_object=policy,
            target_filter={"name": policy},
        ),
        subclass_1=dict(
            aci_class="snmpCommunityP",
            aci_rn="community-{0}".format(community),
            module_object=community,
            target_filter={"name": community},
        ),
    )

    aci.get_existing()

    if state == "present":
        aci.payload(
            aci_class="snmpCommunityP",
            class_config=dict(name=community, descr=description),
        )

        aci.get_diff(aci_class="snmpCommunityP")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Copyright 2025 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The nxos hsrp_interfaces fact class
It is in this file the configuration is collected from the device
for a given resource, parsed, and the facts tree is populated
based on the configuration.
"""

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common import (
    utils,
)

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.argspec.hsrp_interfaces.hsrp_interfaces import (
    Hsrp_interfacesArgs,
)
from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.rm_templates.hsrp_interfaces import (
    Hsrp_interfacesTemplate,
)


class Hsrp_interfacesFacts(object):
    """The nxos hsrp_interfaces facts class"""

    def __init__(self, module, subspec="config", options="options"):
        self._module = module
        self.argument_spec = Hsrp_interfacesArgs.argument_spec

    def get_hsrp_data(self, connection):
        return connection.get("show running-config | section ^interface")

    def flatten_config(self, config):
        dataLines = config.split("\n")
        finalConfig = []
        hsrp_line = ""
        for line in dataLines:
            if (
                line.startswith("  hsrp ")
                and len(line.split()) == 2
                and (line.split()[1]).isdigit()
                or line.startswith("interface ")
            ):
                if line.startswith("  hsrp "):
                    hsrp_line = line
                finalConfig.append(line)
                if line.startswith("interface "):
                    hsrp_line = ""
            else:
                if not line.startswith("hsrp "):
                    finalConfig.append(hsrp_line + line)
        return "\n".join(finalConfig)

    def add_defaults(self, intf_conf):
        if intf_conf.get("standby_options") or intf_conf.get("standby"):
            if intf_conf.get("standby"):
                if not intf_conf["standby"].get("version"):
                    intf_conf["standby"]["version"] = 1
            else:
                intf_conf["standby"] = {"version": 1}

        if "standby_options" in intf_conf:
            for stdby in intf_conf["standby_options"]:
                if not stdby.get("priority"):
                    stdby["priority"] = {"level": 100}
        return intf_conf

    def handle_grp_options(self, objs):

        hsrp_objs = []
        for cf in objs:
            hsrp_conf = []

            intf_conf = {
                "name": cf.get("name"),
            }
            if cf.get("standby"):
                intf_conf["standby"] = cf.get("standby")

            for grp, standby_options in cf.items():
                if grp.startswith("group_"):
                    hsrp_conf.append(standby_options)
            if hsrp_conf:
                intf_conf["standby_options"] = hsrp_conf
            intf_conf = self.add_defaults(intf_conf)
            hsrp_objs.append(intf_conf)
        return hsrp_objs

    def populate_facts(self, connection, ansible_facts, data=None):
        """Populate the facts for Hsrp_interfaces network resource

        :param connection: the device connection
        :param ansible_facts: Facts dictionary
        :param data: previously collected conf

        :rtype: dictionary
        :returns: facts
        """
        facts = {}
        objs = []

        if not data:
            data = self.get_hsrp_data(connection)

        data = self.flatten_config(data)
        # parse native config using the Hsrp_interfaces template
        hsrp_interfaces_parser = Hsrp_interfacesTemplate(
            lines=data.splitlines(),
            module=self._module,
        )
        objs = list(hsrp_interfaces_parser.parse().values())

        hsrp_objs = self.handle_grp_options(objs)

        ansible_facts["ansible_network_resources"].pop("hsrp_interfaces", None)
        params = utils.remove_empties(
            hsrp_interfaces_parser.validate_config(
                self.argument_spec,
                {"config": hsrp_objs},
                redact=True,
            ),
        )
        facts["hsrp_interfaces"] = params.get("config", [])
        ansible_facts["ansible_network_resources"].update(facts)

        return ansible_facts
